@@ -20,6 +20,7 @@ import fastmcp
 from fastmcp.cli import run as run_module
 from fastmcp.cli.install import install_app
 from fastmcp.server.server import FastMCP
+from fastmcp.utilities.cli import is_already_in_uv_subprocess, load_and_merge_config
 from fastmcp.utilities.inspect import (
     InspectFormat,
     format_info,
@@ -193,7 +194,6 @@ async def dev(
     Args:
         server_spec: Python file to run, optionally with :object suffix, or None to auto-detect fastmcp.json
     """
-    from fastmcp.utilities.cli import load_and_merge_config
 
     try:
         # Load config and apply CLI overrides
@@ -415,7 +415,6 @@ async def run(
     Args:
         server_spec: Python file, object specification (file:obj), config file, URL, or None to auto-detect
     """
-    from fastmcp.utilities.cli import is_already_in_uv_subprocess, load_and_merge_config
 
     # Check if we were spawned by uv (or user explicitly set --skip-env)
     if skip_env or is_already_in_uv_subprocess():
@@ -446,6 +445,9 @@ async def run(
     final_path = path or config.deployment.path
     final_log_level = log_level or config.deployment.log_level
     final_server_args = server_args or config.deployment.args
+    # Use CLI override if provided, otherwise use settings
+    # no_banner CLI flag overrides the show_cli_banner setting
+    final_no_banner = no_banner if no_banner else not fastmcp.settings.show_cli_banner
 
     logger.debug(
         "Running server or client",
@@ -466,35 +468,53 @@ async def run(
     needs_uv = config.environment.build_command(test_cmd) != test_cmd and not skip_env
 
     if needs_uv:
-        # Use uv run subprocess - always use run_with_uv which handles output correctly
+        # Build the inner fastmcp command
+        inner_cmd = ["fastmcp", "run", server_spec]
+
+        # Add transport options to the inner command
+        if final_transport:
+            inner_cmd.extend(["--transport", final_transport])
+        # Only add HTTP-specific options for non-stdio transports
+        if final_transport != "stdio":
+            if final_host:
+                inner_cmd.extend(["--host", final_host])
+            if final_port:
+                inner_cmd.extend(["--port", str(final_port)])
+            if final_path:
+                inner_cmd.extend(["--path", final_path])
+        if final_log_level:
+            inner_cmd.extend(["--log-level", final_log_level])
+        if final_no_banner:
+            inner_cmd.append("--no-banner")
+        # Add skip-env flag to prevent infinite recursion
+        inner_cmd.append("--skip-env")
+
+        # Add server args if any
+        if final_server_args:
+            inner_cmd.append("--")
+            inner_cmd.extend(final_server_args)
+
+        # Build the full uv command using the config's environment
+        cmd = config.environment.build_command(inner_cmd)
+
+        # Set marker to prevent infinite loops when subprocess calls FastMCP again
+        env = os.environ | {"FASTMCP_UV_SPAWNED": "1"}
+
+        # Run the command
+        logger.debug(f"Running command: {' '.join(cmd)}")
         try:
-            run_module.run_with_uv(
-                server_spec=server_spec,
-                python_version=config.environment.python,
-                with_packages=config.environment.dependencies,
-                with_requirements=Path(config.environment.requirements)
-                if config.environment.requirements
-                else None,
-                project=Path(config.environment.project)
-                if config.environment.project
-                else None,
-                transport=final_transport,
-                host=final_host,
-                port=final_port,
-                path=final_path,
-                log_level=final_log_level,
-                show_banner=not no_banner,
-                editable=config.environment.editable,
-            )
-        except Exception as e:
+            process = subprocess.run(cmd, check=True, env=env)
+            sys.exit(process.returncode)
+        except subprocess.CalledProcessError as e:
             logger.error(
                 f"Failed to run: {e}",
                 extra={
                     "server_spec": server_spec,
                     "error": str(e),
+                    "returncode": e.returncode,
                 },
             )
-            sys.exit(1)
+            sys.exit(e.returncode)
     else:
         # Use direct import for backwards compatibility
         try:
@@ -506,7 +526,7 @@ async def run(
                 path=final_path,
                 log_level=final_log_level,
                 server_args=list(final_server_args) if final_server_args else [],
-                show_banner=not no_banner,
+                show_banner=not final_no_banner,
                 skip_source=skip_source,
             )
         except Exception as e:
@@ -598,7 +618,6 @@ async def inspect(
     Args:
         server_spec: Python file to inspect, optionally with :object suffix, or fastmcp.json
     """
-    from fastmcp.utilities.cli import is_already_in_uv_subprocess, load_and_merge_config
 
     # Check if we were spawned by uv (or user explicitly set --skip-env)
     if skip_env or is_already_in_uv_subprocess():
@@ -641,6 +660,7 @@ async def inspect(
             "fastmcp",
             "inspect",
             server_spec,
+            "--skip-env",  # Prevent infinite recursion
         ]
 
         # Add format and output flags if specified
