@@ -7,6 +7,7 @@ from .transports import (
     PythonStdioTransport,
     NodeStdioTransport,
     UvxStdioTransport,
+    UvStdioTransport,
     NpxStdioTransport,
     FastMCPTransport,
     StreamableHttpTransport,
@@ -22,6 +23,7 @@ __all__ = [
     "PythonStdioTransport",
     "NodeStdioTransport",
     "UvxStdioTransport",
+    "UvStdioTransport",
     "NpxStdioTransport",
     "FastMCPTransport",
     "StreamableHttpTransport",
