@@ -1,10 +1,9 @@
-from .resource import Resource
+from .resource import FunctionResource, Resource
 from .template import ResourceTemplate
 from .types import (
     BinaryResource,
     DirectoryResource,
     FileResource,
-    FunctionResource,
     HttpResource,
     TextResource,
 )
