@@ -1,7 +1,6 @@
 from __future__ import annotations
 
 from typing import Any
-from urllib.parse import urljoin
 
 from mcp.server.auth.middleware.auth_context import AuthContextMiddleware
 from mcp.server.auth.middleware.bearer_auth import (
@@ -146,8 +145,9 @@ class AuthProvider(TokenVerifierProtocol):
             return None
 
         if path:
-            return AnyHttpUrl(urljoin(str(self.base_url), path))
-
+            prefix = str(self.base_url).rstrip("/")
+            suffix = path.lstrip("/")
+            return AnyHttpUrl(f"{prefix}/{suffix}")
         return self.base_url
 
 
@@ -353,10 +353,16 @@ class OAuthProvider(
 
         # Add protected resource routes if this server is also acting as a resource server
         if resource_url:
+            supported_scopes = (
+                self.client_registration_options.valid_scopes
+                if self.client_registration_options
+                and self.client_registration_options.valid_scopes
+                else self.required_scopes
+            )
             protected_routes = create_protected_resource_routes(
                 resource_url=resource_url,
                 authorization_servers=[self.issuer_url],
-                scopes_supported=self.required_scopes,
+                scopes_supported=supported_scopes,
             )
             oauth_routes.extend(protected_routes)
 
