@@ -16,7 +16,131 @@ def default_serializer(data: Any) -> str:
     return pydantic_core.to_json(data, fallback=str).decode()
 
 
-class LoggingMiddleware(Middleware):
+class BaseLoggingMiddleware(Middleware):
+    """Base class for logging middleware."""
+
+    logger: Logger
+    log_level: int
+    include_payloads: bool
+    include_payload_length: bool
+    estimate_payload_tokens: bool
+    max_payload_length: int | None
+    methods: list[str] | None
+    structured_logging: bool
+    payload_serializer: Callable[[Any], str] | None
+
+    def _serialize_payload(self, context: MiddlewareContext[Any]) -> str:
+        payload: str
+
+        if not self.payload_serializer:
+            payload = default_serializer(context.message)
+        else:
+            try:
+                payload = self.payload_serializer(context.message)
+            except Exception as e:
+                self.logger.warning(
+                    f"Failed to serialize payload due to {e}: {context.type} {context.method} {context.source}."
+                )
+                payload = default_serializer(context.message)
+
+        return payload
+
+    def _format_message(self, message: dict[str, str | int]) -> str:
+        """Format a message for logging."""
+        if self.structured_logging:
+            return json.dumps(message)
+        else:
+            return " ".join([f"{k}={v}" for k, v in message.items()])
+
+    def _get_timestamp_from_context(self, context: MiddlewareContext[Any]) -> str:
+        """Get a timestamp from the context."""
+        return context.timestamp.isoformat()
+
+    def _create_before_message(
+        self, context: MiddlewareContext[Any], event: str
+    ) -> dict[str, str | int]:
+        message = self._create_base_message(context, event)
+
+        if (
+            self.include_payloads
+            or self.include_payload_length
+            or self.estimate_payload_tokens
+        ):
+            payload = self._serialize_payload(context)
+
+            if self.include_payload_length or self.estimate_payload_tokens:
+                payload_length = len(payload)
+                payload_tokens = payload_length // 4
+                if self.estimate_payload_tokens:
+                    message["payload_tokens"] = payload_tokens
+                if self.include_payload_length:
+                    message["payload_length"] = payload_length
+
+            if self.max_payload_length and len(payload) > self.max_payload_length:
+                payload = payload[: self.max_payload_length] + "..."
+
+            if self.include_payloads:
+                message["payload"] = payload
+                message["payload_type"] = type(context.message).__name__
+
+        return message
+
+    def _create_after_message(
+        self, context: MiddlewareContext[Any], event: str
+    ) -> dict[str, str | int]:
+        return self._create_base_message(context, event)
+
+    def _create_base_message(
+        self,
+        context: MiddlewareContext[Any],
+        event: str,
+    ) -> dict[str, str | int]:
+        """Format a message for logging."""
+
+        parts: dict[str, str | int] = {
+            "event": event,
+            "timestamp": self._get_timestamp_from_context(context),
+            "method": context.method or "unknown",
+            "type": context.type,
+            "source": context.source,
+        }
+
+        return parts
+
+    async def on_message(
+        self, context: MiddlewareContext[Any], call_next: CallNext[Any, Any]
+    ) -> Any:
+        """Log all messages."""
+
+        if self.methods and context.method not in self.methods:
+            return await call_next(context)
+
+        request_start_log_message = self._create_before_message(
+            context, "request_start"
+        )
+
+        formatted_message = self._format_message(request_start_log_message)
+        self.logger.log(self.log_level, f"Processing message: {formatted_message}")
+
+        try:
+            result = await call_next(context)
+
+            request_success_log_message = self._create_after_message(
+                context, "request_success"
+            )
+
+            formatted_message = self._format_message(request_success_log_message)
+            self.logger.log(self.log_level, f"Completed message: {formatted_message}")
+
+            return result
+        except Exception as e:
+            self.logger.log(
+                logging.ERROR, f"Failed message: {context.method or 'unknown'} - {e}"
+            )
+            raise
+
+
+class LoggingMiddleware(BaseLoggingMiddleware):
     """Middleware that provides comprehensive request and response logging.
 
     Logs all MCP messages with configurable detail levels. Useful for debugging,
@@ -37,9 +161,12 @@ class LoggingMiddleware(Middleware):
 
     def __init__(
         self,
+        *,
         logger: logging.Logger | None = None,
         log_level: int = logging.INFO,
         include_payloads: bool = False,
+        include_payload_length: bool = False,
+        estimate_payload_tokens: bool = False,
         max_payload_length: int = 1000,
         methods: list[str] | None = None,
         payload_serializer: Callable[[Any], str] | None = None,
@@ -50,68 +177,25 @@ class LoggingMiddleware(Middleware):
             logger: Logger instance to use. If None, creates a logger named 'fastmcp.requests'
             log_level: Log level for messages (default: INFO)
             include_payloads: Whether to include message payloads in logs
+            include_payload_length: Whether to include response size in logs
+            estimate_payload_tokens: Whether to estimate response tokens
             max_payload_length: Maximum length of payload to log (prevents huge logs)
             methods: List of methods to log. If None, logs all methods.
+            payload_serializer: Callable that converts objects to a JSON string for the
+                payload. If not provided, uses FastMCP's default tool serializer.
         """
         self.logger: Logger = logger or logging.getLogger("fastmcp.requests")
-        self.log_level: int = log_level
+        self.log_level = log_level
         self.include_payloads: bool = include_payloads
+        self.include_payload_length: bool = include_payload_length
+        self.estimate_payload_tokens: bool = estimate_payload_tokens
         self.max_payload_length: int = max_payload_length
         self.methods: list[str] | None = methods
         self.payload_serializer: Callable[[Any], str] | None = payload_serializer
-
-    def _format_message(self, context: MiddlewareContext[Any]) -> str:
-        """Format a message for logging."""
-        parts = [
-            f"source={context.source}",
-            f"type={context.type}",
-            f"method={context.method or 'unknown'}",
-        ]
-
-        if self.include_payloads:
-            payload: str
-
-            if not self.payload_serializer:
-                payload = default_serializer(context.message)
-            else:
-                try:
-                    payload = self.payload_serializer(context.message)
-                except Exception as e:
-                    self.logger.warning(
-                        f"Failed {e} to serialize payload: {context.type} {context.method} {context.source}."
-                    )
-                    payload = default_serializer(context.message)
-
-            if len(payload) > self.max_payload_length:
-                payload = payload[: self.max_payload_length] + "..."
-
-            parts.append(f"payload={payload}")
-        return " ".join(parts)
-
-    async def on_message(
-        self, context: MiddlewareContext[Any], call_next: CallNext[Any, Any]
-    ) -> Any:
-        """Log all messages."""
-        message_info = self._format_message(context)
-        if self.methods and context.method not in self.methods:
-            return await call_next(context)
-
-        self.logger.log(self.log_level, f"Processing message: {message_info}")
-
-        try:
-            result = await call_next(context)
-            self.logger.log(
-                self.log_level, f"Completed message: {context.method or 'unknown'}"
-            )
-            return result
-        except Exception as e:
-            self.logger.log(
-                logging.ERROR, f"Failed message: {context.method or 'unknown'} - {e}"
-            )
-            raise
+        self.structured_logging: bool = False
 
 
-class StructuredLoggingMiddleware(Middleware):
+class StructuredLoggingMiddleware(BaseLoggingMiddleware):
     """Middleware that provides structured JSON logging for better log analysis.
 
     Outputs structured logs that are easier to parse and analyze with log
@@ -129,9 +213,12 @@ class StructuredLoggingMiddleware(Middleware):
 
     def __init__(
         self,
+        *,
         logger: logging.Logger | None = None,
         log_level: int = logging.INFO,
         include_payloads: bool = False,
+        include_payload_length: bool = False,
+        estimate_payload_tokens: bool = False,
         methods: list[str] | None = None,
         payload_serializer: Callable[[Any], str] | None = None,
     ):
@@ -141,74 +228,18 @@ class StructuredLoggingMiddleware(Middleware):
             logger: Logger instance to use. If None, creates a logger named 'fastmcp.structured'
             log_level: Log level for messages (default: INFO)
             include_payloads: Whether to include message payloads in logs
+            include_payload_length: Whether to include payload size in logs
+            estimate_payload_tokens: Whether to estimate token count using length // 4
             methods: List of methods to log. If None, logs all methods.
-            serializer: Callable that converts objects to a JSON string for the
+            payload_serializer: Callable that converts objects to a JSON string for the
                 payload. If not provided, uses FastMCP's default tool serializer.
         """
         self.logger: Logger = logger or logging.getLogger("fastmcp.structured")
         self.log_level: int = log_level
         self.include_payloads: bool = include_payloads
+        self.include_payload_length: bool = include_payload_length
+        self.estimate_payload_tokens: bool = estimate_payload_tokens
         self.methods: list[str] | None = methods
         self.payload_serializer: Callable[[Any], str] | None = payload_serializer
-
-    def _create_log_entry(
-        self, context: MiddlewareContext[Any], event: str, **extra_fields: Any
-    ) -> dict[str, Any]:
-        """Create a structured log entry."""
-        entry = {
-            "event": event,
-            "timestamp": context.timestamp.isoformat(),
-            "source": context.source,
-            "type": context.type,
-            "method": context.method,
-            **extra_fields,
-        }
-
-        if self.include_payloads:
-            payload: str
-
-            if not self.payload_serializer:
-                payload = default_serializer(context.message)
-            else:
-                try:
-                    payload = self.payload_serializer(context.message)
-                except Exception as e:
-                    self.logger.warning(
-                        f"Failed {str(e)} to serialize payload: {context.type} {context.method} {context.source}."
-                    )
-                    payload = default_serializer(context.message)
-
-            entry["payload"] = payload
-
-        return entry
-
-    async def on_message(
-        self, context: MiddlewareContext[Any], call_next: CallNext[Any, Any]
-    ) -> Any:
-        """Log structured message information."""
-        start_entry = self._create_log_entry(context, "request_start")
-        if self.methods and context.method not in self.methods:
-            return await call_next(context)
-
-        self.logger.log(self.log_level, json.dumps(start_entry))
-
-        try:
-            result = await call_next(context)
-
-            success_entry = self._create_log_entry(
-                context,
-                "request_success",
-                result_type=type(result).__name__ if result else None,
-            )
-            self.logger.log(self.log_level, json.dumps(success_entry))
-
-            return result
-        except Exception as e:
-            error_entry = self._create_log_entry(
-                context,
-                "request_error",
-                error_type=type(e).__name__,
-                error_message=str(e),
-            )
-            self.logger.log(logging.ERROR, json.dumps(error_entry))
-            raise
+        self.max_payload_length: int | None = None
+        self.structured_logging: bool = True
