@@ -32,6 +32,7 @@ class TestMainCLI:
         """Test parsing invalid environment variables exits."""
         with pytest.raises(SystemExit) as exc_info:
             _parse_env_var("INVALID_FORMAT")
+        assert isinstance(exc_info.value, SystemExit)
         assert exc_info.value.code == 1
 
 
@@ -47,14 +48,14 @@ class TestVersionCommand:
     def test_version_command_parsing(self):
         """Test that the version command parses arguments correctly."""
         command, bound, _ = app.parse_args(["version"])
-        assert command.__name__ == "version"
+        assert command.__name__ == "version"  # type: ignore[attr-defined]
         # Default arguments aren't included in bound.arguments
         assert bound.arguments == {}
 
     def test_version_command_with_copy_flag(self):
         """Test that the version command parses --copy flag correctly."""
         command, bound, _ = app.parse_args(["version", "--copy"])
-        assert command.__name__ == "version"
+        assert command.__name__ == "version"  # type: ignore[attr-defined]
         assert bound.arguments == {"copy": True}
 
     @patch("fastmcp.cli.cli.pyperclip.copy")
@@ -371,6 +372,30 @@ class TestRunCommand:
         assert bound.arguments["project"] == Path("./test-env")
         assert bound.arguments["skip_source"] is True
 
+    def test_show_cli_banner_setting(self):
+        """Test that show_cli_banner setting works with environment variable."""
+        import os
+        from unittest import mock
+
+        from fastmcp.settings import Settings
+
+        # Test default (banner shown)
+        settings = Settings()
+        assert settings.show_cli_banner is True
+
+        # Test with env var set to false (banner hidden)
+        with mock.patch.dict(os.environ, {"FASTMCP_SHOW_CLI_BANNER": "false"}):
+            settings = Settings()
+            assert settings.show_cli_banner is False
+
+        # Test CLI precedence logic (simulated)
+        with mock.patch.dict(os.environ, {"FASTMCP_SHOW_CLI_BANNER": "true"}):
+            settings = Settings()
+            # CLI --no-banner flag would override
+            cli_no_banner = True
+            final = cli_no_banner if cli_no_banner else not settings.show_cli_banner
+            assert final is True  # Banner suppressed by CLI flag
+
 
 class TestWindowsSpecific:
     """Test Windows-specific functionality."""
