@@ -44,7 +44,7 @@ class MockOAuthProvider:
     - Network calls to external services
     """
 
-    def __init__(self, port: int = 9999):
+    def __init__(self, port: int = 0):
         self.port = port
         self.base_url = f"http://localhost:{port}"
         self.app = None
@@ -229,9 +229,20 @@ class MockOAuthProvider:
 
     async def start(self):
         """Start the mock OAuth server."""
+        import socket
+
         from uvicorn import Config, Server
 
         self.app = self.create_app()
+
+        # If port is 0, find an available port
+        if self.port == 0:
+            with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
+                s.bind(("127.0.0.1", 0))
+                s.listen(1)
+                self.port = s.getsockname()[1]
+
+        self.base_url = f"http://localhost:{self.port}"
         config = Config(self.app, host="localhost", port=self.port, log_level="error")
         self.server = Server(config)
 
@@ -239,13 +250,13 @@ class MockOAuthProvider:
         asyncio.create_task(self.server.serve())
 
         # Wait for server to be ready
-        await asyncio.sleep(0.5)
+        await asyncio.sleep(0.05)
 
     async def stop(self):
         """Stop the mock OAuth server."""
         if self.server:
             self.server.should_exit = True
-            await asyncio.sleep(0.1)
+            await asyncio.sleep(0.01)
 
     def reset(self):
         """Reset all state for next test."""
@@ -308,7 +319,7 @@ def oauth_proxy(jwt_verifier):
 @pytest.fixture
 async def mock_oauth_provider():
     """Create and start a mock OAuth provider."""
-    provider = MockOAuthProvider(port=9999)
+    provider = MockOAuthProvider()
     await provider.start()
     yield provider
     await provider.stop()
@@ -365,6 +376,7 @@ class TestOAuthProxyInitialization:
         assert proxy._redirect_path == "/custom/callback"
         assert proxy._forward_pkce is False
         assert proxy._token_endpoint_auth_method == "client_secret_post"
+        assert proxy.client_registration_options is not None
         assert proxy.client_registration_options.valid_scopes == ["custom", "scopes"]
 
     def test_redirect_path_normalization(self, jwt_verifier):
@@ -394,8 +406,8 @@ class TestOAuthProxyClientRegistration:
 
         await oauth_proxy.register_client(client_info)
 
-        # Client should be stored with original credentials
-        stored = oauth_proxy._clients.get("original-client")
+        # Client should be retrievable with original credentials
+        stored = await oauth_proxy.get_client("original-client")
         assert stored is not None
         assert stored.client_id == "original-client"
         assert stored.client_secret == "original-secret"
@@ -793,3 +805,171 @@ class TestOAuthProxyE2E:
         txn_id = query_params["state"][0]
         transaction = proxy._oauth_transactions[txn_id]
         assert "proxy_code_verifier" in transaction
+
+
+class TestParameterForwarding:
+    """Tests for forwarding custom parameters to upstream OAuth provider."""
+
+    @pytest.fixture
+    def proxy_with_extra_params(self, jwt_verifier):
+        """Create OAuthProxy with extra parameters configured."""
+        return OAuthProxy(
+            upstream_authorization_endpoint="https://oauth.example.com/authorize",
+            upstream_token_endpoint="https://oauth.example.com/token",
+            upstream_client_id="upstream-client",
+            upstream_client_secret="upstream-secret",
+            token_verifier=jwt_verifier,
+            base_url="https://proxy.example.com",
+            extra_authorize_params={"audience": "https://api.example.com"},
+            extra_token_params={"audience": "https://api.example.com"},
+        )
+
+    @pytest.fixture
+    def proxy_without_extra_params(self, jwt_verifier):
+        """Create OAuthProxy without extra parameters."""
+        return OAuthProxy(
+            upstream_authorization_endpoint="https://oauth.example.com/authorize",
+            upstream_token_endpoint="https://oauth.example.com/token",
+            upstream_client_id="upstream-client",
+            upstream_client_secret="upstream-secret",
+            token_verifier=jwt_verifier,
+            base_url="https://proxy.example.com",
+        )
+
+    async def test_resource_parameter_forwarding(self, proxy_without_extra_params):
+        """Test that RFC 8707 resource parameter is forwarded from client request."""
+        client = OAuthClientInformationFull(
+            client_id="test-client",
+            client_secret="test-secret",
+            redirect_uris=[AnyUrl("http://localhost:12345/callback")],
+        )
+
+        params = AuthorizationParams(
+            redirect_uri=AnyUrl("http://localhost:12345/callback"),
+            redirect_uri_provided_explicitly=True,
+            state="client-state",
+            code_challenge="client_challenge",
+            scopes=["read"],
+            resource="https://api.example.com/v1",  # RFC 8707 resource indicator
+        )
+
+        redirect_url = await proxy_without_extra_params.authorize(client, params)
+        query_params = parse_qs(urlparse(redirect_url).query)
+
+        # Resource parameter should be forwarded to upstream
+        assert "resource" in query_params
+        assert query_params["resource"][0] == "https://api.example.com/v1"
+
+    async def test_extra_authorize_params(self, proxy_with_extra_params):
+        """Test that extra authorization parameters are included."""
+        client = OAuthClientInformationFull(
+            client_id="test-client",
+            client_secret="test-secret",
+            redirect_uris=[AnyUrl("http://localhost:12345/callback")],
+        )
+
+        params = AuthorizationParams(
+            redirect_uri=AnyUrl("http://localhost:12345/callback"),
+            redirect_uri_provided_explicitly=True,
+            state="client-state",
+            code_challenge="client_challenge",
+            scopes=["read"],
+        )
+
+        redirect_url = await proxy_with_extra_params.authorize(client, params)
+        query_params = parse_qs(urlparse(redirect_url).query)
+
+        # Extra audience parameter should be included
+        assert "audience" in query_params
+        assert query_params["audience"][0] == "https://api.example.com"
+
+    async def test_resource_and_extra_params_together(self, proxy_with_extra_params):
+        """Test that both resource and extra params can be used together."""
+        client = OAuthClientInformationFull(
+            client_id="test-client",
+            client_secret="test-secret",
+            redirect_uris=[AnyUrl("http://localhost:12345/callback")],
+        )
+
+        params = AuthorizationParams(
+            redirect_uri=AnyUrl("http://localhost:12345/callback"),
+            redirect_uri_provided_explicitly=True,
+            state="client-state",
+            code_challenge="client_challenge",
+            scopes=["read"],
+            resource="https://resource.example.com",  # Client-specified resource
+        )
+
+        redirect_url = await proxy_with_extra_params.authorize(client, params)
+        query_params = parse_qs(urlparse(redirect_url).query)
+
+        # Both resource and audience should be present
+        assert "resource" in query_params
+        assert query_params["resource"][0] == "https://resource.example.com"
+        assert "audience" in query_params
+        assert query_params["audience"][0] == "https://api.example.com"
+
+    async def test_no_extra_params_when_not_configured(
+        self, proxy_without_extra_params
+    ):
+        """Test that no extra params are added when not configured."""
+        client = OAuthClientInformationFull(
+            client_id="test-client",
+            client_secret="test-secret",
+            redirect_uris=[AnyUrl("http://localhost:12345/callback")],
+        )
+
+        params = AuthorizationParams(
+            redirect_uri=AnyUrl("http://localhost:12345/callback"),
+            redirect_uri_provided_explicitly=True,
+            state="client-state",
+            code_challenge="client_challenge",
+            scopes=["read"],
+            # No resource parameter
+        )
+
+        redirect_url = await proxy_without_extra_params.authorize(client, params)
+        query_params = parse_qs(urlparse(redirect_url).query)
+
+        # No audience parameter should be present (not configured)
+        assert "audience" not in query_params
+        # No resource parameter should be present (not provided by client)
+        assert "resource" not in query_params
+
+    async def test_multiple_extra_params(self, jwt_verifier):
+        """Test multiple extra parameters can be configured and forwarded."""
+        proxy = OAuthProxy(
+            upstream_authorization_endpoint="https://oauth.example.com/authorize",
+            upstream_token_endpoint="https://oauth.example.com/token",
+            upstream_client_id="upstream-client",
+            upstream_client_secret="upstream-secret",
+            token_verifier=jwt_verifier,
+            base_url="https://proxy.example.com",
+            extra_authorize_params={
+                "audience": "https://api.example.com",
+                "prompt": "consent",
+                "max_age": "3600",
+            },
+        )
+
+        client = OAuthClientInformationFull(
+            client_id="test-client",
+            client_secret="test-secret",
+            redirect_uris=[AnyUrl("http://localhost:12345/callback")],
+        )
+
+        params = AuthorizationParams(
+            redirect_uri=AnyUrl("http://localhost:12345/callback"),
+            redirect_uri_provided_explicitly=True,
+            state="client-state",
+            code_challenge="client_challenge",
+            scopes=["read"],
+        )
+
+        redirect_url = await proxy.authorize(client, params)
+        query_params = parse_qs(urlparse(redirect_url).query)
+
+        # All extra parameters should be included
+        assert query_params["audience"][0] == "https://api.example.com"
+        assert query_params["prompt"][0] == "consent"
+        assert query_params["max_age"][0] == "3600"
