@@ -11,6 +11,7 @@ from fastmcp.server.middleware.error_handling import (
     RetryMiddleware,
 )
 from fastmcp.server.middleware.middleware import MiddlewareContext
+from fastmcp.utilities.tests import caplog_for_fastmcp
 
 
 @pytest.fixture
@@ -60,8 +61,9 @@ class TestErrorHandlingMiddleware:
         middleware = ErrorHandlingMiddleware()
         error = ValueError("test error")
 
-        with caplog.at_level(logging.ERROR):
-            middleware._log_error(error, mock_context)
+        with caplog_for_fastmcp(caplog):
+            with caplog.at_level(logging.ERROR):
+                middleware._log_error(error, mock_context)
 
         assert "Error in test_method: ValueError: test error" in caplog.text
         assert "ValueError:test_method" in middleware.error_counts
@@ -72,8 +74,9 @@ class TestErrorHandlingMiddleware:
         middleware = ErrorHandlingMiddleware(include_traceback=True)
         error = ValueError("test error")
 
-        with caplog.at_level(logging.ERROR):
-            middleware._log_error(error, mock_context)
+        with caplog_for_fastmcp(caplog):
+            with caplog.at_level(logging.ERROR):
+                middleware._log_error(error, mock_context)
 
         assert "Error in test_method: ValueError: test error" in caplog.text
         # The traceback is added to the log message
@@ -95,8 +98,9 @@ class TestErrorHandlingMiddleware:
         middleware = ErrorHandlingMiddleware(error_callback=callback)
         error = ValueError("test error")
 
-        with caplog.at_level(logging.ERROR):
-            middleware._log_error(error, mock_context)
+        with caplog_for_fastmcp(caplog):
+            with caplog.at_level(logging.ERROR):
+                middleware._log_error(error, mock_context)
 
         assert "Error in error callback: callback error" in caplog.text
 
@@ -189,10 +193,12 @@ class TestErrorHandlingMiddleware:
         middleware = ErrorHandlingMiddleware()
         mock_call_next = AsyncMock(side_effect=ValueError("test error"))
 
-        with caplog.at_level(logging.ERROR):
-            with pytest.raises(McpError) as exc_info:
-                await middleware.on_message(mock_context, mock_call_next)
+        with caplog_for_fastmcp(caplog):
+            with caplog.at_level(logging.ERROR):
+                with pytest.raises(McpError) as exc_info:
+                    await middleware.on_message(mock_context, mock_call_next)
 
+        assert isinstance(exc_info.value, McpError)
         assert exc_info.value.error.code == -32602
         assert "Invalid params: test error" in exc_info.value.error.message
         assert "Error in test_method: ValueError: test error" in caplog.text
@@ -292,8 +298,9 @@ class TestRetryMiddleware:
             ]
         )
 
-        with caplog.at_level(logging.WARNING):
-            result = await middleware.on_request(mock_context, mock_call_next)
+        with caplog_for_fastmcp(caplog):
+            with caplog.at_level(logging.WARNING):
+                result = await middleware.on_request(mock_context, mock_call_next)
 
         assert result == "test_result"
         assert mock_call_next.call_count == 3
@@ -306,9 +313,10 @@ class TestRetryMiddleware:
         # Fail all attempts
         mock_call_next = AsyncMock(side_effect=ConnectionError("connection failed"))
 
-        with caplog.at_level(logging.WARNING):
-            with pytest.raises(ConnectionError):
-                await middleware.on_request(mock_context, mock_call_next)
+        with caplog_for_fastmcp(caplog):
+            with caplog.at_level(logging.WARNING):
+                with pytest.raises(ConnectionError):
+                    await middleware.on_request(mock_context, mock_call_next)
 
         assert mock_call_next.call_count == 3  # initial + 2 retries
         assert "Retrying in" in caplog.text
@@ -384,14 +392,19 @@ class TestErrorHandlingMiddlewareIntegration:
 
         error_handling_server.add_middleware(ErrorHandlingMiddleware())
 
-        with caplog.at_level(logging.ERROR):
-            async with Client(error_handling_server) as client:
-                # Test different types of errors
-                with pytest.raises(Exception):
-                    await client.call_tool("failing_operation", {"error_type": "value"})
+        with caplog_for_fastmcp(caplog):
+            with caplog.at_level(logging.ERROR):
+                async with Client(error_handling_server) as client:
+                    # Test different types of errors
+                    with pytest.raises(Exception):
+                        await client.call_tool(
+                            "failing_operation", {"error_type": "value"}
+                        )
 
-                with pytest.raises(Exception):
-                    await client.call_tool("failing_operation", {"error_type": "file"})
+                    with pytest.raises(Exception):
+                        await client.call_tool(
+                            "failing_operation", {"error_type": "file"}
+                        )
 
         log_text = caplog.text
 
@@ -442,17 +455,20 @@ class TestErrorHandlingMiddlewareIntegration:
 
         error_handling_server.add_middleware(ErrorHandlingMiddleware())
 
-        with caplog.at_level(logging.ERROR):
-            async with Client(error_handling_server) as client:
-                # Successful operation (should not generate error logs)
-                await client.call_tool("reliable_operation", {"data": "test"})
+        with caplog_for_fastmcp(caplog):
+            with caplog.at_level(logging.ERROR):
+                async with Client(error_handling_server) as client:
+                    # Successful operation (should not generate error logs)
+                    await client.call_tool("reliable_operation", {"data": "test"})
 
-                # Failed operation (should generate error log)
-                with pytest.raises(Exception):
-                    await client.call_tool("failing_operation", {"error_type": "value"})
+                    # Failed operation (should generate error log)
+                    with pytest.raises(Exception):
+                        await client.call_tool(
+                            "failing_operation", {"error_type": "value"}
+                        )
 
-                # Another successful operation
-                await client.call_tool("reliable_operation", {"data": "test2"})
+                    # Another successful operation
+                    await client.call_tool("reliable_operation", {"data": "test2"})
 
         log_text = caplog.text
 
@@ -532,18 +548,19 @@ class TestRetryMiddlewareIntegration:
             )
         )
 
-        with caplog.at_level(logging.WARNING):
-            async with Client(error_handling_server) as client:
-                # This operation fails intermittently - try several times
-                success_count = 0
-                for _ in range(5):
-                    try:
-                        await client.call_tool(
-                            "intermittent_operation", {"fail_rate": 0.7}
-                        )
-                        success_count += 1
-                    except Exception:
-                        pass  # Some failures expected even with retries
+        with caplog_for_fastmcp(caplog):
+            with caplog.at_level(logging.WARNING):
+                async with Client(error_handling_server) as client:
+                    # This operation fails intermittently - try several times
+                    success_count = 0
+                    for _ in range(5):
+                        try:
+                            await client.call_tool(
+                                "intermittent_operation", {"fail_rate": 0.7}
+                            )
+                            success_count += 1
+                        except Exception:
+                            pass  # Some failures expected even with retries
 
         # Should have some retry log messages
         # Note: Retry logs might not appear if the underlying errors are wrapped by FastMCP
@@ -583,17 +600,22 @@ class TestRetryMiddlewareIntegration:
             )
         )
 
-        with caplog.at_level(logging.ERROR):
-            async with Client(error_handling_server) as client:
-                # Try intermittent operation
-                try:
-                    await client.call_tool("intermittent_operation", {"fail_rate": 0.9})
-                except Exception:
-                    pass  # May still fail even with retries
+        with caplog_for_fastmcp(caplog):
+            with caplog.at_level(logging.ERROR):
+                async with Client(error_handling_server) as client:
+                    # Try intermittent operation
+                    try:
+                        await client.call_tool(
+                            "intermittent_operation", {"fail_rate": 0.9}
+                        )
+                    except Exception:
+                        pass  # May still fail even with retries
 
-                # Try permanent failure
-                with pytest.raises(Exception):
-                    await client.call_tool("failing_operation", {"error_type": "value"})
+                    # Try permanent failure
+                    with pytest.raises(Exception):
+                        await client.call_tool(
+                            "failing_operation", {"error_type": "value"}
+                        )
 
         log_text = caplog.text
 
