@@ -3,27 +3,42 @@
 import datetime
 import json
 import logging
+import re
 from typing import Any, Literal, TypeVar
 from unittest.mock import AsyncMock, MagicMock
 
 import mcp
+import mcp.types
 import pytest
 from inline_snapshot import snapshot
 from pydantic import AnyUrl
 
+from fastmcp import FastMCP
+from fastmcp.client import Client
 from fastmcp.resources.template import ResourceTemplate
 from fastmcp.server.middleware.logging import (
     LoggingMiddleware,
     StructuredLoggingMiddleware,
 )
 from fastmcp.server.middleware.middleware import CallNext, MiddlewareContext
-from fastmcp.server.server import FastMCP
+from fastmcp.utilities.tests import caplog_for_fastmcp
 
 FIXED_DATE = datetime.datetime(2023, 1, 1, tzinfo=datetime.timezone.utc)
 
 T = TypeVar("T")
 
 
+def remove_line_numbers(logs: str) -> str:
+    """Remove line numbers from log messages."""
+    trimmed_logs = ""
+    lines = logs.split("\n")
+    for line in lines:
+        # Match only the first `:\d+ `
+        line = re.sub(pattern=r":\d+ ", repl=":LINE_NUMBER ", string=line, count=1)
+        trimmed_logs += line + "\n"
+    return trimmed_logs
+
+
 def new_mock_context(
     message: T,
     method: str | None = None,
@@ -56,188 +71,185 @@ def mock_context():
 
 
 @pytest.fixture
-def mock_call_next():
+def mock_call_next() -> AsyncMock:
     """Create a mock call_next function."""
     return AsyncMock(return_value="test_result")
 
 
-class TestLoggingMiddleware:
-    """Test logging middleware functionality."""
-
-    def test_init_default(self):
-        """Test default initialization."""
-        middleware = LoggingMiddleware()
-        assert middleware.logger.name == "fastmcp.requests"
-        assert middleware.log_level == logging.INFO
-        assert middleware.include_payloads is False
-        assert middleware.max_payload_length == 1000
-
-    def test_init_custom(self):
-        """Test custom initialization."""
-        logger = logging.getLogger("custom")
-        middleware = LoggingMiddleware(
-            logger=logger,
-            log_level=logging.DEBUG,
-            include_payloads=True,
-            max_payload_length=500,
-        )
-        assert middleware.logger is logger
-        assert middleware.log_level == logging.DEBUG
-        assert middleware.include_payloads is True
-        assert middleware.max_payload_length == 500
-
-    def test_format_message_without_payloads(
-        self, mock_context: MiddlewareContext[Any]
-    ):
-        """Test message formatting without payloads."""
-        middleware = LoggingMiddleware()
-        formatted = middleware._format_message(mock_context)
-
-        assert "source=client" in formatted
-        assert "type=request" in formatted
-        assert "method=test_method" in formatted
-        assert "payload=" not in formatted
-
-    def test_format_message_with_payloads(self, mock_context: MiddlewareContext[Any]):
-        """Test message formatting with payloads."""
-        middleware = LoggingMiddleware(include_payloads=True)
-        formatted = middleware._format_message(mock_context)
-
-        assert formatted == snapshot(
-            'source=client type=request method=test_method payload={"method":"tools/call","params":{"_meta":null,"name":"test_method","arguments":{"param":"value"}}}'
-        )
-
-    def test_format_message_long_payload(self, mock_context: MiddlewareContext[Any]):
-        """Test message formatting with long payload truncation."""
-        middleware = LoggingMiddleware(include_payloads=True, max_payload_length=10)
-        formatted = middleware._format_message(mock_context)
-
-        assert "payload=" in formatted
-        assert "..." in formatted
-
-    async def test_on_message_success(
-        self,
-        mock_context: MiddlewareContext[Any],
-        mock_call_next: CallNext[Any, Any],
-        caplog: pytest.LogCaptureFixture,
-    ):
-        """Test logging successful messages."""
-        middleware = LoggingMiddleware()
-
-        with caplog.at_level(logging.INFO):
-            result = await middleware.on_message(mock_context, mock_call_next)
-
-        assert result == "test_result"
-        assert mock_call_next.called
-        assert "Processing message:" in caplog.text
-        assert "Completed message: test_method" in caplog.text
-
-    async def test_on_message_failure(
-        self, mock_context: MiddlewareContext[Any], caplog: pytest.LogCaptureFixture
-    ):
-        """Test logging failed messages."""
-        middleware = LoggingMiddleware()
-        mock_call_next = AsyncMock(side_effect=ValueError("test error"))
-
-        with caplog.at_level(logging.INFO):
-            with pytest.raises(ValueError):
-                await middleware.on_message(mock_context, mock_call_next)
-
-        assert "Processing message:" in caplog.text
-        assert "Failed message: test_method - test error" in caplog.text
-
-
 class TestStructuredLoggingMiddleware:
     """Test structured logging middleware functionality."""
 
     def test_init_default(self):
         """Test default initialization."""
-        middleware = StructuredLoggingMiddleware()
-        assert middleware.logger.name == "fastmcp.structured"
+        middleware = LoggingMiddleware()
+
+        assert middleware.logger.name == "fastmcp.requests"
         assert middleware.log_level == logging.INFO
         assert middleware.include_payloads is False
+        assert middleware.max_payload_length == 1000
+        assert middleware.include_payload_length is False
+        assert middleware.estimate_payload_tokens is False
+        assert middleware.structured_logging is False
 
-    def test_create_log_entry_basic(self, mock_context: MiddlewareContext[Any]):
-        """Test creating basic log entry."""
-        middleware = StructuredLoggingMiddleware()
-        entry = middleware._create_log_entry(mock_context, "test_event")
-
-        assert entry == snapshot(
-            {
-                "event": "test_event",
-                "timestamp": "2023-01-01T00:00:00+00:00",
-                "source": "client",
-                "type": "request",
-                "method": "test_method",
-            }
+    def test_init_custom(self):
+        """Test custom initialization."""
+        logger = logging.getLogger("custom")
+        middleware = StructuredLoggingMiddleware(
+            logger=logger,
+            log_level=logging.DEBUG,
+            include_payloads=True,
+            include_payload_length=False,
+            estimate_payload_tokens=True,
         )
+        assert middleware.logger is logger
+        assert middleware.log_level == logging.DEBUG
+        assert middleware.include_payloads is True
+        assert middleware.include_payload_length is False
+        assert middleware.estimate_payload_tokens is True
 
-    def test_create_log_entry_with_payload(self, mock_context: MiddlewareContext[Any]):
-        """Test creating log entry with payload."""
-        middleware = StructuredLoggingMiddleware(include_payloads=True)
-        entry = middleware._create_log_entry(mock_context, "test_event")
+    class TestHelperMethods:
+        def test_create_before_message(self, mock_context: MiddlewareContext[Any]):
+            """Test message formatting without payloads."""
+            middleware = StructuredLoggingMiddleware()
 
-        assert entry == snapshot(
-            {
-                "event": "test_event",
-                "timestamp": "2023-01-01T00:00:00+00:00",
-                "source": "client",
-                "type": "request",
-                "method": "test_method",
-                "payload": '{"method":"tools/call","params":{"_meta":null,"name":"test_method","arguments":{"param":"value"}}}',
-            }
-        )
+            message = middleware._create_before_message(mock_context, "test_event")
 
-    def test_create_log_entry_with_extra_fields(
-        self, mock_context: MiddlewareContext[Any]
-    ):
-        """Test creating log entry with extra fields."""
-        middleware = StructuredLoggingMiddleware()
-        entry = middleware._create_log_entry(
-            mock_context, "test_event", extra_field="extra_value"
-        )
+            assert message == snapshot(
+                {
+                    "event": "test_event",
+                    "timestamp": "2023-01-01T00:00:00+00:00",
+                    "source": "client",
+                    "type": "request",
+                    "method": "test_method",
+                }
+            )
 
-        assert entry["extra_field"] == "extra_value"
+        def test_create_message_with_payloads(
+            self, mock_context: MiddlewareContext[Any]
+        ):
+            """Test message formatting with payloads."""
+            middleware = StructuredLoggingMiddleware(include_payloads=True)
+
+            message = middleware._create_before_message(mock_context, "test_event")
+
+            assert message == snapshot(
+                {
+                    "event": "test_event",
+                    "timestamp": "2023-01-01T00:00:00+00:00",
+                    "source": "client",
+                    "type": "request",
+                    "method": "test_method",
+                    "payload": '{"method":"tools/call","params":{"_meta":null,"name":"test_method","arguments":{"param":"value"}}}',
+                    "payload_type": "CallToolRequest",
+                }
+            )
+
+        def test_calculate_response_size(self, mock_context: MiddlewareContext[Any]):
+            """Test response size calculation."""
+            middleware = StructuredLoggingMiddleware(include_payload_length=True)
+            message = middleware._create_before_message(mock_context, "test_event")
+
+            assert message == snapshot(
+                {
+                    "event": "test_event",
+                    "timestamp": "2023-01-01T00:00:00+00:00",
+                    "source": "client",
+                    "type": "request",
+                    "method": "test_method",
+                    "payload_length": 98,
+                }
+            )
+
+        def test_calculate_response_size_with_token_estimation(
+            self, mock_context: MiddlewareContext[Any]
+        ):
+            """Test response size calculation with token estimation."""
+            middleware = StructuredLoggingMiddleware(
+                include_payload_length=True, estimate_payload_tokens=True
+            )
+            message = middleware._create_before_message(mock_context, "test_event")
+
+            assert message == snapshot(
+                {
+                    "event": "test_event",
+                    "timestamp": "2023-01-01T00:00:00+00:00",
+                    "source": "client",
+                    "type": "request",
+                    "method": "test_method",
+                    "payload_tokens": 24,
+                    "payload_length": 98,
+                }
+            )
 
     async def test_on_message_success(
         self,
         mock_context: MiddlewareContext[Any],
-        mock_call_next: CallNext[Any, Any],
         caplog: pytest.LogCaptureFixture,
     ):
-        """Test structured logging of successful messages."""
+        """Test logging successful messages."""
         middleware = StructuredLoggingMiddleware()
+        mock_call_next = AsyncMock(return_value="test_result")
 
-        with caplog.at_level(logging.INFO):
+        with caplog_for_fastmcp(caplog):
             result = await middleware.on_message(mock_context, mock_call_next)
 
         assert result == "test_result"
+        assert mock_call_next.called
+        assert remove_line_numbers(caplog.text) == snapshot("""\
+INFO     fastmcp.structured:logging.py:LINE_NUMBER Processing message: {"event": "request_start", "timestamp": "2023-01-01T00:00:00+00:00", "method": "test_method", "type": "request", "source": "client"}
+INFO     fastmcp.structured:logging.py:LINE_NUMBER Completed message: {"event": "request_success", "timestamp": "2023-01-01T00:00:00+00:00", "method": "test_method", "type": "request", "source": "client"}
 
-        # Check that we have structured JSON logs
-        log_lines = [record.message for record in caplog.records]
+""")
 
-        assert len(log_lines) == 2  # start and success entries
+    async def test_on_message_failure(
+        self, mock_context: MiddlewareContext[Any], caplog: pytest.LogCaptureFixture
+    ):
+        """Test logging failed messages."""
+        middleware = StructuredLoggingMiddleware()
+        mock_call_next = AsyncMock(side_effect=ValueError("test error"))
 
-        assert json.loads(log_lines[0]) == snapshot(
-            {
-                "event": "request_start",
-                "timestamp": "2023-01-01T00:00:00+00:00",
-                "source": "client",
-                "type": "request",
-                "method": "test_method",
-            }
+        with caplog_for_fastmcp(caplog):
+            with pytest.raises(ValueError):
+                await middleware.on_message(mock_context, mock_call_next)
+
+        assert "Processing message:" in caplog.text
+        assert "Failed message: test_method - test error" in caplog.text
+
+
+class TestLoggingMiddleware:
+    """Test structured logging middleware functionality."""
+
+    def test_init_default(self):
+        """Test default initialization."""
+        middleware = LoggingMiddleware()
+        assert middleware.logger.name == "fastmcp.requests"
+        assert middleware.log_level == logging.INFO
+        assert middleware.include_payloads is False
+        assert middleware.include_payload_length is False
+        assert middleware.estimate_payload_tokens is False
+
+    def test_format_message(self, mock_context: MiddlewareContext[Any]):
+        """Test message formatting."""
+        middleware = LoggingMiddleware()
+        message = middleware._create_before_message(mock_context, "test_event")
+        formatted = middleware._format_message(message)
+
+        assert formatted == snapshot(
+            "event=test_event timestamp=2023-01-01T00:00:00+00:00 method=test_method type=request source=client"
         )
 
-        assert json.loads(log_lines[1]) == snapshot(
-            {
-                "event": "request_success",
-                "timestamp": "2023-01-01T00:00:00+00:00",
-                "source": "client",
-                "type": "request",
-                "method": "test_method",
-                "result_type": "str",
-            }
-        )
+    def test_create_before_message_long_payload(
+        self, mock_context: MiddlewareContext[Any]
+    ):
+        """Test message formatting with long payload truncation."""
+        middleware = LoggingMiddleware(include_payloads=True, max_payload_length=10)
+
+        message = middleware._create_before_message(mock_context, "test_event")
+
+        formatted = middleware._format_message(message)
+
+        assert "payload=" in formatted
+        assert "..." in formatted
 
     async def test_on_message_failure(
         self, mock_context: MiddlewareContext[Any], caplog: pytest.LogCaptureFixture
@@ -246,7 +258,7 @@ class TestStructuredLoggingMiddleware:
         middleware = StructuredLoggingMiddleware()
         mock_call_next = AsyncMock(side_effect=ValueError("test error"))
 
-        with caplog.at_level(logging.INFO):
+        with caplog_for_fastmcp(caplog):
             with pytest.raises(ValueError):
                 await middleware.on_message(mock_context, mock_call_next)
 
@@ -254,20 +266,15 @@ class TestStructuredLoggingMiddleware:
         log_lines = [record.message for record in caplog.records]
         assert len(log_lines) == 2  # start and error entries
 
-        start_entry = json.loads(log_lines[0])
+        # Extract JSON from "Processing message: {JSON}"
+        start_message = log_lines[0]
+        assert start_message.startswith("Processing message: ")
+        start_json = start_message[len("Processing message: ") :]
+        start_entry = json.loads(start_json)
         assert start_entry["event"] == "request_start"
 
-        assert json.loads(log_lines[1]) == snapshot(
-            {
-                "event": "request_error",
-                "timestamp": "2023-01-01T00:00:00+00:00",
-                "source": "client",
-                "type": "request",
-                "method": "test_method",
-                "error_type": "ValueError",
-                "error_message": "test error",
-            }
-        )
+        # Error messages have different format - check the second log entry
+        assert "Failed message:" in log_lines[1]
 
     async def test_on_message_with_pydantic_types_in_payload(
         self,
@@ -287,7 +294,7 @@ class TestStructuredLoggingMiddleware:
 
         middleware = StructuredLoggingMiddleware(include_payloads=True)
 
-        with caplog.at_level(logging.INFO):
+        with caplog_for_fastmcp(caplog):
             result = await middleware.on_message(mock_context, mock_call_next)
 
         assert result == "test_result"
@@ -295,7 +302,12 @@ class TestStructuredLoggingMiddleware:
         log_lines = [record.message for record in caplog.records]
 
         assert len(log_lines) == 2
-        assert json.loads(log_lines[0]) == snapshot(
+
+        # Extract JSON from log messages
+        start_message = log_lines[0]
+        assert start_message.startswith("Processing message: ")
+        start_json = start_message[len("Processing message: ") :]
+        assert json.loads(start_json) == snapshot(
             {
                 "event": "request_start",
                 "timestamp": "2023-01-01T00:00:00+00:00",
@@ -303,17 +315,20 @@ class TestStructuredLoggingMiddleware:
                 "type": "request",
                 "method": "test_method",
                 "payload": '{"method":"resources/read","params":{"_meta":null,"uri":"test://example/1"}}',
+                "payload_type": "ReadResourceRequest",
             }
         )
-        assert json.loads(log_lines[1]) == snapshot(
+
+        success_message = log_lines[1]
+        assert success_message.startswith("Completed message: ")
+        success_json = success_message[len("Completed message: ") :]
+        assert json.loads(success_json) == snapshot(
             {
                 "event": "request_success",
                 "timestamp": "2023-01-01T00:00:00+00:00",
                 "source": "client",
                 "type": "request",
                 "method": "test_method",
-                "result_type": "str",
-                "payload": '{"method":"resources/read","params":{"_meta":null,"uri":"test://example/1"}}',
             }
         )
 
@@ -334,14 +349,19 @@ class TestStructuredLoggingMiddleware:
 
         middleware = StructuredLoggingMiddleware(include_payloads=True)
 
-        with caplog.at_level(logging.INFO):
+        with caplog_for_fastmcp(caplog):
             result = await middleware.on_message(mock_context, mock_call_next)
 
         assert result == "test_result"
 
         log_lines = [record.message for record in caplog.records]
         assert len(log_lines) == 2
-        assert json.loads(log_lines[0]) == snapshot(
+
+        # Extract JSON from log message
+        start_message = log_lines[0]
+        assert start_message.startswith("Processing message: ")
+        start_json = start_message[len("Processing message: ") :]
+        assert json.loads(start_json) == snapshot(
             {
                 "event": "request_start",
                 "timestamp": "2023-01-01T00:00:00+00:00",
@@ -349,6 +369,7 @@ class TestStructuredLoggingMiddleware:
                 "type": "request",
                 "method": "test_method",
                 "payload": '{"name":"tmpl","title":null,"description":null,"tags":[],"meta":null,"enabled":true,"uri_template":"tmpl://{id}","mime_type":"text/plain","parameters":{"id":{"type":"string"}},"annotations":null}',
+                "payload_type": "ResourceTemplate",
             }
         )
 
@@ -373,14 +394,19 @@ class TestStructuredLoggingMiddleware:
 
         middleware = StructuredLoggingMiddleware(include_payloads=True)
 
-        with caplog.at_level(logging.INFO):
+        with caplog_for_fastmcp(caplog):
             result = await middleware.on_message(mock_context, mock_call_next)
 
         assert result == "test_result"
 
         log_lines = [record.message for record in caplog.records]
         assert len(log_lines) >= 2
-        assert json.loads(log_lines[0]) == snapshot(
+
+        # Extract JSON from log message
+        start_message = log_lines[0]
+        assert start_message.startswith("Processing message: ")
+        start_json = start_message[len("Processing message: ") :]
+        assert json.loads(start_json) == snapshot(
             {
                 "event": "request_start",
                 "timestamp": "2023-01-01T00:00:00+00:00",
@@ -388,6 +414,7 @@ class TestStructuredLoggingMiddleware:
                 "type": "request",
                 "method": "test_method",
                 "payload": '{"method":"tools/call","params":{"_meta":null,"name":"test_method","arguments":{"obj":"NON_SERIALIZABLE"}}}',
+                "payload_type": "CallToolRequest",
             }
         )
 
@@ -414,14 +441,19 @@ class TestStructuredLoggingMiddleware:
             include_payloads=True, payload_serializer=custom_serializer
         )
 
-        with caplog.at_level(logging.INFO):
+        with caplog_for_fastmcp(caplog):
             result = await middleware.on_message(mock_context, mock_call_next)
 
         assert result == "test_result"
 
         log_lines = [record.message for record in caplog.records]
         assert len(log_lines) >= 2
-        assert json.loads(log_lines[0]) == snapshot(
+
+        # Extract JSON from log message
+        start_message = log_lines[0]
+        assert start_message.startswith("Processing message: ")
+        start_json = start_message[len("Processing message: ") :]
+        assert json.loads(start_json) == snapshot(
             {
                 "event": "request_start",
                 "timestamp": "2023-01-01T00:00:00+00:00",
@@ -429,6 +461,7 @@ class TestStructuredLoggingMiddleware:
                 "type": "request",
                 "method": "test_method",
                 "payload": "CUSTOM_PAYLOAD",
+                "payload_type": "CallToolRequest",
             }
         )
 
@@ -473,42 +506,81 @@ def logging_server():
 class TestLoggingMiddlewareIntegration:
     """Integration tests for logging middleware with real FastMCP server."""
 
+    @pytest.fixture
+    def logging_server(self):
+        """Create a FastMCP server specifically for logging middleware tests."""
+        mcp = FastMCP("LoggingTestServer")
+
+        @mcp.tool
+        def simple_operation(data: str) -> str:
+            """A simple operation for testing logging."""
+            return f"Processed: {data}"
+
+        @mcp.tool
+        def complex_operation(items: list[str], mode: str = "default") -> dict:
+            """A complex operation with structured data."""
+            return {"processed_items": len(items), "mode": mode, "result": "success"}
+
+        @mcp.tool
+        def operation_with_error(should_fail: bool = False) -> str:
+            """An operation that can be made to fail."""
+            if should_fail:
+                raise ValueError("Operation failed intentionally")
+            return "Operation completed successfully"
+
+        @mcp.resource("log://test")
+        def test_resource() -> str:
+            """A test resource for logging."""
+            return "Test resource content"
+
+        @mcp.prompt
+        def test_prompt() -> str:
+            """A test prompt for logging."""
+            return "Test prompt content"
+
+        return mcp
+
     async def test_logging_middleware_logs_successful_operations(
         self, logging_server: FastMCP, caplog: pytest.LogCaptureFixture
     ):
         """Test that logging middleware captures successful operations."""
-        from fastmcp.client import Client
+        logging_middleware = LoggingMiddleware(methods=["tools/call"])
+        logging_middleware._get_timestamp_from_context = (  # ty: ignore[invalid-assignment]
+            lambda _: FIXED_DATE.isoformat()
+        )
 
-        logging_server.add_middleware(LoggingMiddleware(methods=["tools/call"]))
+        logging_server.add_middleware(logging_middleware)
 
-        with caplog.at_level(logging.INFO):
-            async with Client(logging_server) as client:
-                await client.call_tool("simple_operation", {"data": "test_data"})
-                await client.call_tool(
-                    "complex_operation", {"items": ["a", "b", "c"], "mode": "batch"}
-                )
-
-        log_text = caplog.text
+        with caplog_for_fastmcp(caplog):
+            with caplog.at_level(logging.INFO):
+                async with Client(logging_server) as client:
+                    await client.call_tool(
+                        name="simple_operation", arguments={"data": "test_data"}
+                    )
+                    await client.call_tool(
+                        name="complex_operation",
+                        arguments={"items": ["a", "b", "c"], "mode": "batch"},
+                    )
 
         # Should have processing and completion logs for both operations
-        assert "Processing message:" in log_text
-        assert "Completed message: tools/call" in log_text
+        assert remove_line_numbers(caplog.text) == snapshot("""\
+INFO     mcp.server.lowlevel.server:server.py:LINE_NUMBER Processing request of type CallToolRequest
+INFO     fastmcp.requests:logging.py:LINE_NUMBER Processing message: event=request_start timestamp=2023-01-01T00:00:00+00:00 method=tools/call type=request source=client
+INFO     fastmcp.requests:logging.py:LINE_NUMBER Completed message: event=request_success timestamp=2023-01-01T00:00:00+00:00 method=tools/call type=request source=client
+INFO     mcp.server.lowlevel.server:server.py:LINE_NUMBER Processing request of type ListToolsRequest
+INFO     mcp.server.lowlevel.server:server.py:LINE_NUMBER Processing request of type CallToolRequest
+INFO     fastmcp.requests:logging.py:LINE_NUMBER Processing message: event=request_start timestamp=2023-01-01T00:00:00+00:00 method=tools/call type=request source=client
+INFO     fastmcp.requests:logging.py:LINE_NUMBER Completed message: event=request_success timestamp=2023-01-01T00:00:00+00:00 method=tools/call type=request source=client
 
-        # Should have captured both tool calls
-        processing_count = log_text.count("Processing message:")
-        completion_count = log_text.count("Completed message:")
-        assert processing_count == 2
-        assert completion_count == 2
+""")
 
     async def test_logging_middleware_logs_failures(
         self, logging_server: FastMCP, caplog: pytest.LogCaptureFixture
     ):
         """Test that logging middleware captures failed operations."""
-        from fastmcp.client import Client
-
         logging_server.add_middleware(LoggingMiddleware(methods=["tools/call"]))
 
-        with caplog.at_level(logging.INFO):
+        with caplog_for_fastmcp(caplog):
             async with Client(logging_server) as client:
                 # This should fail and be logged
                 with pytest.raises(Exception):
@@ -526,39 +598,56 @@ class TestLoggingMiddlewareIntegration:
         self, logging_server: FastMCP, caplog: pytest.LogCaptureFixture
     ):
         """Test logging middleware when configured to include payloads."""
-        from fastmcp.client import Client
 
-        logging_server.add_middleware(
-            LoggingMiddleware(
-                include_payloads=True, max_payload_length=500, methods=["tools/call"]
-            )
+        middleware = LoggingMiddleware(
+            include_payloads=True, max_payload_length=500, methods=["tools/call"]
         )
+        middleware._get_timestamp_from_context = (  # ty: ignore[invalid-assignment]
+            lambda _: FIXED_DATE.isoformat()
+        )
+        logging_server.add_middleware(middleware)
 
-        with caplog.at_level(logging.INFO):
+        with caplog_for_fastmcp(caplog):
             async with Client(logging_server) as client:
                 await client.call_tool("simple_operation", {"data": "payload_test"})
 
         log_text = caplog.text
 
-        # Should include payload information
-        assert "Processing message:" in log_text
-        assert "payload=" in log_text
+        # Remove client IDs from log text for consistent snapshots
+        import re
+
+        log_text = re.sub(r"\[Client-[^\]]+\]", "[Client-XXXX]", log_text)
+
+        assert remove_line_numbers(log_text) == snapshot("""\
+DEBUG    fastmcp.fastmcp.client.transports:transports.py:LINE_NUMBER Inferred transport: <FastMCPTransport(server='LoggingTestServer')>
+DEBUG    fastmcp.fastmcp.client.client:client.py:LINE_NUMBER [Client-XXXX] called call_tool: simple_operation
+DEBUG    fastmcp.fastmcp.server.server:server.py:LINE_NUMBER [LoggingTestServer] Handler called: list_tools
+DEBUG    fastmcp.fastmcp.server.server:server.py:LINE_NUMBER [LoggingTestServer] Handler called: call_tool simple_operation with {'data': 'payload_test'}
+INFO     fastmcp.requests:logging.py:LINE_NUMBER Processing message: event=request_start timestamp=2023-01-01T00:00:00+00:00 method=tools/call type=request source=client payload={"_meta":null,"name":"simple_operation","arguments":{"data":"payload_test"}} payload_type=CallToolRequestParams
+INFO     fastmcp.requests:logging.py:LINE_NUMBER Completed message: event=request_success timestamp=2023-01-01T00:00:00+00:00 method=tools/call type=request source=client
+DEBUG    fastmcp.fastmcp.server.server:server.py:LINE_NUMBER [LoggingTestServer] Handler called: list_tools
+
+""")
 
     async def test_structured_logging_middleware_produces_json(
         self, logging_server: FastMCP, caplog: pytest.LogCaptureFixture
     ):
         """Test that structured logging middleware produces parseable JSON logs."""
-        import json
 
-        from fastmcp.client import Client
-
-        logging_server.add_middleware(
-            StructuredLoggingMiddleware(include_payloads=True, methods=["tools/call"])
+        logging_middleware = StructuredLoggingMiddleware(
+            include_payloads=True, methods=["tools/call"]
+        )
+        logging_middleware._get_timestamp_from_context = (  # ty: ignore[invalid-assignment]
+            lambda _: FIXED_DATE.isoformat()
         )
 
-        with caplog.at_level(logging.INFO):
+        logging_server.add_middleware(logging_middleware)
+
+        with caplog_for_fastmcp(caplog):
             async with Client(logging_server) as client:
-                await client.call_tool("simple_operation", {"data": "json_test"})
+                await client.call_tool(
+                    name="simple_operation", arguments={"data": "json_test"}
+                )
 
         # Extract JSON log entries
         log_lines = [
@@ -569,61 +658,59 @@ class TestLoggingMiddlewareIntegration:
 
         assert len(log_lines) >= 2  # Should have start and success entries
 
-        # Each log line should be valid JSON
-        for line in log_lines:
-            log_entry = json.loads(line)
-            assert "event" in log_entry
-            assert "timestamp" in log_entry
-            assert "source" in log_entry
-            assert "type" in log_entry
-            assert "method" in log_entry
+        # Remove client IDs from log text for consistent snapshots
+        import re
+
+        log_text = re.sub(r"\[Client-[^\]]+\]", "[Client-XXXX]", caplog.text)
+
+        assert remove_line_numbers(log_text) == snapshot("""\
+DEBUG    fastmcp.fastmcp.client.transports:transports.py:LINE_NUMBER Inferred transport: <FastMCPTransport(server='LoggingTestServer')>
+DEBUG    fastmcp.fastmcp.client.client:client.py:LINE_NUMBER [Client-XXXX] called call_tool: simple_operation
+DEBUG    fastmcp.fastmcp.server.server:server.py:LINE_NUMBER [LoggingTestServer] Handler called: list_tools
+DEBUG    fastmcp.fastmcp.server.server:server.py:LINE_NUMBER [LoggingTestServer] Handler called: call_tool simple_operation with {'data': 'json_test'}
+INFO     fastmcp.structured:logging.py:LINE_NUMBER Processing message: {"event": "request_start", "timestamp": "2023-01-01T00:00:00+00:00", "method": "tools/call", "type": "request", "source": "client", "payload": "{\\"_meta\\":null,\\"name\\":\\"simple_operation\\",\\"arguments\\":{\\"data\\":\\"json_test\\"}}", "payload_type": "CallToolRequestParams"}
+INFO     fastmcp.structured:logging.py:LINE_NUMBER Completed message: {"event": "request_success", "timestamp": "2023-01-01T00:00:00+00:00", "method": "tools/call", "type": "request", "source": "client"}
+DEBUG    fastmcp.fastmcp.server.server:server.py:LINE_NUMBER [LoggingTestServer] Handler called: list_tools
+
+""")
 
     async def test_structured_logging_middleware_handles_errors(
         self, logging_server: FastMCP, caplog: pytest.LogCaptureFixture
     ):
         """Test structured logging of errors with JSON format."""
-        import json
 
-        from fastmcp.client import Client
-
-        logging_server.add_middleware(
-            StructuredLoggingMiddleware(methods=["tools/call"])
+        logging_middleware = StructuredLoggingMiddleware(methods=["tools/call"])
+        logging_middleware._get_timestamp_from_context = (  # ty: ignore[invalid-assignment]
+            lambda _: FIXED_DATE.isoformat()
         )
 
-        with caplog.at_level(logging.INFO):
-            async with Client(logging_server) as client:
-                with pytest.raises(Exception):
-                    await client.call_tool(
-                        "operation_with_error", {"should_fail": True}
-                    )
+        logging_server.add_middleware(logging_middleware)
 
-        # Extract JSON log entries
-        log_lines = [
-            record.message
-            for record in caplog.records
-            if record.name == "fastmcp.structured"
-        ]
+        with caplog_for_fastmcp(caplog):
+            with caplog.at_level(logging.INFO):
+                async with Client(logging_server) as client:
+                    with pytest.raises(Exception):
+                        await client.call_tool(
+                            "operation_with_error", {"should_fail": True}
+                        )
 
-        # Should have start and error entries
-        assert len(log_lines) >= 2
+        # Verify that the structured logging middleware properly logs errors
+        logs = caplog.text
 
-        # Find the error entry
-        error_entries = []
-        for line in log_lines:
-            log_entry = json.loads(line)
-            if log_entry.get("event") == "request_error":
-                error_entries.append(log_entry)
+        # The key assertion: structured logging middleware logged the error in JSON format
+        assert re.search(
+            r"fastmcp\.structured.*Failed message: tools/call.*Operation failed intentionally",
+            logs,
+        )
 
-        assert len(error_entries) == 1
-        error_entry = error_entries[0]
-        assert "error_type" in error_entry
-        assert "error_message" in error_entry
+        # Verify the error contains expected error type and message
+        assert "ValueError" in logs
+        assert "Operation failed intentionally" in logs
 
     async def test_logging_middleware_with_different_operations(
         self, logging_server: FastMCP, caplog: pytest.LogCaptureFixture
     ):
         """Test logging middleware with various MCP operations."""
-        from fastmcp.client import Client
 
         logging_server.add_middleware(
             LoggingMiddleware(
@@ -636,7 +723,7 @@ class TestLoggingMiddlewareIntegration:
             )
         )
 
-        with caplog.at_level(logging.INFO):
+        with caplog_for_fastmcp(caplog):
             async with Client(logging_server) as client:
                 # Test different operation types
                 await client.call_tool("simple_operation", {"data": "test"})
@@ -662,8 +749,6 @@ class TestLoggingMiddlewareIntegration:
         import io
         import logging
 
-        from fastmcp.client import Client
-
         # Create custom logger
         log_buffer = io.StringIO()
         handler = logging.StreamHandler(log_buffer)
