@@ -8,6 +8,7 @@ from pathlib import Path
 from typing import Annotated, Any, Literal
 
 import pytest
+from inline_snapshot import snapshot
 from mcp import McpError
 from mcp.types import (
     AudioContent,
@@ -21,6 +22,7 @@ from pydantic import AnyUrl, BaseModel, Field, TypeAdapter
 from typing_extensions import TypedDict
 
 from fastmcp import Client, Context, FastMCP
+from fastmcp.client.client import CallToolResult
 from fastmcp.client.transports import FastMCPTransport
 from fastmcp.exceptions import ToolError
 from fastmcp.prompts.prompt import Prompt, PromptMessage
@@ -185,6 +187,8 @@ class TestTools:
     async def test_tool_returns_list(self, tool_server: FastMCP):
         async with Client(tool_server) as client:
             result = await client.call_tool("list_tool", {})
+            # Adjacent non-MCP list items are combined into single content block
+            assert len(result.content) == 1
             assert result.content[0].text == '["x",2]'  # type: ignore[attr-defined]
             assert result.data == ["x", 2]
 
@@ -426,7 +430,7 @@ class TestToolReturnTypes:
         self, tool_server: FastMCP, tmp_path: Path
     ):
         """Test that lists containing Image objects and other types are handled
-        correctly. Note that the non-MCP content will be grouped together."""
+        correctly. Items now preserve their original order."""
         # Create a test image
         image_path = tmp_path / "test.png"
         image_path.write_bytes(b"test image data")
@@ -435,26 +439,30 @@ class TestToolReturnTypes:
             result = await client.call_tool(
                 "mixed_list_fn", {"image_path": str(image_path)}
             )
-            assert len(result.content) == 3
-            # Check text conversion
+            assert len(result.content) == 4  # Now each item is separate
+            # Check text message (first item)
             content1 = result.content[0]
             assert isinstance(content1, TextContent)
-            assert json.loads(content1.text) == ["text message", {"key": "value"}]
-            # Check image conversion
+            assert content1.text == "text message"
+            # Check image conversion (second item)
             content2 = result.content[1]
             assert isinstance(content2, ImageContent)
             assert content2.mimeType == "image/png"
             assert base64.b64decode(content2.data) == b"test image data"
-            # Check direct TextContent
+            # Check dict content (third item)
             content3 = result.content[2]
             assert isinstance(content3, TextContent)
-            assert content3.text == "direct content"
+            assert json.loads(content3.text) == {"key": "value"}
+            # Check direct TextContent (fourth item)
+            content4 = result.content[3]
+            assert isinstance(content4, TextContent)
+            assert content4.text == "direct content"
 
     async def test_tool_mixed_list_with_audio(
         self, tool_server: FastMCP, tmp_path: Path
     ):
         """Test that lists containing Audio objects and other types are handled
-        correctly. Note that the non-MCP content will be grouped together."""
+        correctly. Items now preserve their original order."""
         # Create a test audio file
         audio_path = tmp_path / "test.wav"
         audio_path.write_bytes(b"test audio data")
@@ -463,26 +471,30 @@ class TestToolReturnTypes:
             result = await client.call_tool(
                 "mixed_audio_list_fn", {"audio_path": str(audio_path)}
             )
-            assert len(result.content) == 3
-            # Check text conversion
+            assert len(result.content) == 4  # Now each item is separate
+            # Check text message (first item)
             content1 = result.content[0]
             assert isinstance(content1, TextContent)
-            assert json.loads(content1.text) == ["text message", {"key": "value"}]
-            # Check audio conversion
+            assert content1.text == "text message"
+            # Check audio conversion (second item)
             content2 = result.content[1]
             assert isinstance(content2, AudioContent)
             assert content2.mimeType == "audio/wav"
             assert base64.b64decode(content2.data) == b"test audio data"
-            # Check direct TextContent
+            # Check dict content (third item)
             content3 = result.content[2]
             assert isinstance(content3, TextContent)
-            assert content3.text == "direct content"
+            assert json.loads(content3.text) == {"key": "value"}
+            # Check direct TextContent (fourth item)
+            content4 = result.content[3]
+            assert isinstance(content4, TextContent)
+            assert content4.text == "direct content"
 
     async def test_tool_mixed_list_with_file(
         self, tool_server: FastMCP, tmp_path: Path
     ):
         """Test that lists containing File objects and other types are handled
-        correctly. Note that the non-MCP content will be grouped together."""
+        correctly. Items now preserve their original order."""
         # Create a test file
         file_path = tmp_path / "test.bin"
         file_path.write_bytes(b"test file data")
@@ -491,12 +503,12 @@ class TestToolReturnTypes:
             result = await client.call_tool(
                 "mixed_file_list_fn", {"file_path": str(file_path)}
             )
-            assert len(result.content) == 3
-            # Check text conversion
+            assert len(result.content) == 4  # Now each item is separate
+            # Check text message (first item)
             content1 = result.content[0]
             assert isinstance(content1, TextContent)
-            assert json.loads(content1.text) == ["text message", {"key": "value"}]
-            # Check file conversion
+            assert content1.text == "text message"
+            # Check file conversion (second item)
             content2 = result.content[1]
             assert isinstance(content2, EmbeddedResource)
             assert content2.type == "resource"
@@ -505,10 +517,14 @@ class TestToolReturnTypes:
             assert hasattr(resource, "blob")
             blob_data = getattr(resource, "blob")
             assert base64.b64decode(blob_data) == b"test file data"
-            # Check direct TextContent
+            # Check dict content (third item)
             content3 = result.content[2]
             assert isinstance(content3, TextContent)
-            assert content3.text == "direct content"
+            assert json.loads(content3.text) == {"key": "value"}
+            # Check direct TextContent (fourth item)
+            content4 = result.content[3]
+            assert isinstance(content4, TextContent)
+            assert content4.text == "direct content"
 
 
 class TestToolParameters:
@@ -919,12 +935,13 @@ class TestToolOutputSchema:
             assert len(tools) == 1
 
             type_schema = TypeAdapter(annotation).json_schema()
+            # Remove title fields from the schema for comparison (title pruning is enabled)
+            type_schema = compress_schema(type_schema, prune_titles=True)
             # this line will fail until MCP adds output schemas!!
             assert tools[0].outputSchema == {
                 "type": "object",
-                "properties": {"result": {**type_schema, "title": "Result"}},
+                "properties": {"result": type_schema},
                 "required": ["result"],
-                "title": "_WrappedResult",
                 "x-fastmcp-wrap-result": True,
             }
 
@@ -942,7 +959,9 @@ class TestToolOutputSchema:
         async with Client(mcp) as client:
             tools = await client.list_tools()
 
-            type_schema = compress_schema(TypeAdapter(annotation).json_schema())
+            type_schema = compress_schema(
+                TypeAdapter(annotation).json_schema(), prune_titles=True
+            )
             assert len(tools) == 1
 
             # Normalize anyOf ordering for comparison since union type order
@@ -1055,9 +1074,8 @@ class TestToolOutputSchema:
             tool = next(t for t in tools if t.name == "primitive_tool")
             expected_schema = {
                 "type": "object",
-                "properties": {"result": {"type": "string", "title": "Result"}},
+                "properties": {"result": {"type": "string"}},
                 "required": ["result"],
-                "title": "_WrappedResult",
                 "x-fastmcp-wrap-result": True,
             }
             assert tool.outputSchema == expected_schema
@@ -1079,12 +1097,13 @@ class TestToolOutputSchema:
             # List tools and verify schema shows wrapped array
             tools = await client.list_tools()
             tool = next(t for t in tools if t.name == "complex_tool")
-            expected_inner_schema = TypeAdapter(list[dict[str, int]]).json_schema()
+            expected_inner_schema = compress_schema(
+                TypeAdapter(list[dict[str, int]]).json_schema(), prune_titles=True
+            )
             expected_schema = {
                 "type": "object",
-                "properties": {"result": {**expected_inner_schema, "title": "Result"}},
+                "properties": {"result": expected_inner_schema},
                 "required": ["result"],
-                "title": "_WrappedResult",
                 "x-fastmcp-wrap-result": True,
             }
             assert tool.outputSchema == expected_schema
@@ -1113,7 +1132,9 @@ class TestToolOutputSchema:
             # List tools and verify schema is object type (not wrapped)
             tools = await client.list_tools()
             tool = next(t for t in tools if t.name == "dataclass_tool")
-            expected_schema = compress_schema(TypeAdapter(User).json_schema())
+            expected_schema = compress_schema(
+                TypeAdapter(User).json_schema(), prune_titles=True
+            )
             assert tool.outputSchema == expected_schema
             assert (
                 tool.outputSchema and "x-fastmcp-wrap-result" not in tool.outputSchema
@@ -1143,20 +1164,37 @@ class TestToolOutputSchema:
             result = await client.call_tool("mixed_output", {})
 
             # Should have multiple content blocks
-            assert len(result.content) >= 2
-
-            # Should have structured output with wrapped result
-            expected_data = [
-                "text message",
-                {"structured": "data"},
-                {
-                    "type": "text",
-                    "text": "direct MCP content",
-                    "annotations": None,
-                    "_meta": None,
-                },
-            ]
-            assert result.structured_content == {"result": expected_data}
+            assert result == snapshot(
+                CallToolResult(
+                    content=[
+                        TextContent(type="text", text="text message"),
+                        TextContent(type="text", text='{"structured":"data"}'),
+                        TextContent(type="text", text="direct MCP content"),
+                    ],
+                    structured_content={
+                        "result": [
+                            "text message",
+                            {"structured": "data"},
+                            {
+                                "type": "text",
+                                "text": "direct MCP content",
+                                "annotations": None,
+                                "_meta": None,
+                            },
+                        ]
+                    },
+                    data=[
+                        "text message",
+                        {"structured": "data"},
+                        {
+                            "type": "text",
+                            "text": "direct MCP content",
+                            "annotations": None,
+                            "_meta": None,
+                        },
+                    ],
+                )
+            )
 
     async def test_output_schema_serialization_edge_cases(self):
         """Test edge cases in output schema serialization."""
