@@ -1,4 +1,3 @@
-import json
 from dataclasses import dataclass
 from typing import Annotated, Any
 
@@ -7,8 +6,10 @@ from dirty_equals import HasName
 from inline_snapshot import snapshot
 from mcp.types import (
     AudioContent,
+    BlobResourceContents,
     EmbeddedResource,
     ImageContent,
+    ResourceLink,
     TextContent,
     TextResourceContents,
 )
@@ -39,16 +40,15 @@ class TestToolFromFunction:
                 "enabled": True,
                 "parameters": {
                     "properties": {
-                        "a": {"title": "A", "type": "integer"},
-                        "b": {"title": "B", "type": "integer"},
+                        "a": {"type": "integer"},
+                        "b": {"type": "integer"},
                     },
                     "required": ["a", "b"],
                     "type": "object",
                 },
                 "output_schema": {
-                    "properties": {"result": {"title": "Result", "type": "integer"}},
+                    "properties": {"result": {"type": "integer"}},
                     "required": ["result"],
-                    "title": "_WrappedResult",
                     "type": "object",
                     "x-fastmcp-wrap-result": True,
                 },
@@ -89,14 +89,13 @@ class TestToolFromFunction:
                 "tags": set(),
                 "enabled": True,
                 "parameters": {
-                    "properties": {"url": {"title": "Url", "type": "string"}},
+                    "properties": {"url": {"type": "string"}},
                     "required": ["url"],
                     "type": "object",
                 },
                 "output_schema": {
-                    "properties": {"result": {"title": "Result", "type": "string"}},
+                    "properties": {"result": {"type": "string"}},
                     "required": ["result"],
-                    "title": "_WrappedResult",
                     "type": "object",
                     "x-fastmcp-wrap-result": True,
                 },
@@ -122,16 +121,15 @@ class TestToolFromFunction:
                 "enabled": True,
                 "parameters": {
                     "properties": {
-                        "x": {"title": "X", "type": "integer"},
-                        "y": {"title": "Y", "type": "integer"},
+                        "x": {"type": "integer"},
+                        "y": {"type": "integer"},
                     },
                     "required": ["x", "y"],
                     "type": "object",
                 },
                 "output_schema": {
-                    "properties": {"result": {"title": "Result", "type": "integer"}},
+                    "properties": {"result": {"type": "integer"}},
                     "required": ["result"],
-                    "title": "_WrappedResult",
                     "type": "object",
                     "x-fastmcp-wrap-result": True,
                 },
@@ -156,16 +154,15 @@ class TestToolFromFunction:
                 "enabled": True,
                 "parameters": {
                     "properties": {
-                        "x": {"title": "X", "type": "integer"},
-                        "y": {"title": "Y", "type": "integer"},
+                        "x": {"type": "integer"},
+                        "y": {"type": "integer"},
                     },
                     "required": ["x", "y"],
                     "type": "object",
                 },
                 "output_schema": {
-                    "properties": {"result": {"title": "Result", "type": "integer"}},
+                    "properties": {"result": {"type": "integer"}},
                     "required": ["result"],
-                    "title": "_WrappedResult",
                     "type": "object",
                     "x-fastmcp-wrap-result": True,
                 },
@@ -195,17 +192,16 @@ class TestToolFromFunction:
                     "$defs": {
                         "UserInput": {
                             "properties": {
-                                "name": {"title": "Name", "type": "string"},
-                                "age": {"title": "Age", "type": "integer"},
+                                "name": {"type": "string"},
+                                "age": {"type": "integer"},
                             },
                             "required": ["name", "age"],
-                            "title": "UserInput",
                             "type": "object",
                         }
                     },
                     "properties": {
-                        "user": {"$ref": "#/$defs/UserInput", "title": "User"},
-                        "flag": {"title": "Flag", "type": "boolean"},
+                        "user": {"$ref": "#/$defs/UserInput"},
+                        "flag": {"type": "boolean"},
                     },
                     "required": ["user", "flag"],
                     "type": "object",
@@ -299,8 +295,8 @@ class TestToolFromFunction:
                 "enabled": True,
                 "parameters": {
                     "properties": {
-                        "_a": {"title": "A", "type": "integer"},
-                        "_b": {"title": "B", "type": "integer"},
+                        "_a": {"type": "integer"},
+                        "_b": {"type": "integer"},
                     },
                     "required": ["_a", "_b"],
                     "type": "object",
@@ -347,16 +343,15 @@ class TestToolFromFunction:
                 "enabled": True,
                 "parameters": {
                     "properties": {
-                        "x": {"title": "X", "type": "integer"},
-                        "y": {"title": "Y", "type": "integer"},
+                        "x": {"type": "integer"},
+                        "y": {"type": "integer"},
                     },
                     "required": ["x", "y"],
                     "type": "object",
                 },
                 "output_schema": {
-                    "properties": {"result": {"title": "Result", "type": "integer"}},
+                    "properties": {"result": {"type": "integer"}},
                     "required": ["result"],
-                    "title": "_WrappedResult",
                     "type": "object",
                     "x-fastmcp-wrap-result": True,
                 },
@@ -466,9 +461,8 @@ class TestToolFromFunctionOutputSchema:
             # Non-object types get wrapped
             expected_schema = {
                 "type": "object",
-                "properties": {"result": {**base_schema, "title": "Result"}},
+                "properties": {"result": base_schema},
                 "required": ["result"],
-                "title": "_WrappedResult",
                 "x-fastmcp-wrap-result": True,
             }
             assert tool.output_schema == expected_schema
@@ -494,9 +488,8 @@ class TestToolFromFunctionOutputSchema:
         base_schema = TypeAdapter(annotation).json_schema()
         expected_schema = {
             "type": "object",
-            "properties": {"result": {**base_schema, "title": "Result"}},
+            "properties": {"result": base_schema},
             "required": ["result"],
-            "title": "_WrappedResult",
             "x-fastmcp-wrap-result": True,
         }
         assert tool.output_schema == expected_schema
@@ -544,7 +537,9 @@ class TestToolFromFunctionOutputSchema:
             return Person(name="John", age=30)
 
         tool = Tool.from_function(func)
-        expected_schema = compress_schema(TypeAdapter(Person).json_schema())
+        expected_schema = compress_schema(
+            TypeAdapter(Person).json_schema(), prune_titles=True
+        )
         assert tool.output_schema == expected_schema
 
     async def test_base_model_return_annotation(self):
@@ -560,11 +555,10 @@ class TestToolFromFunctionOutputSchema:
         assert tool.output_schema == snapshot(
             {
                 "properties": {
-                    "name": {"title": "Name", "type": "string"},
-                    "age": {"title": "Age", "type": "integer"},
+                    "name": {"type": "string"},
+                    "age": {"type": "integer"},
                 },
                 "required": ["name", "age"],
-                "title": "Person",
                 "type": "object",
             }
         )
@@ -581,11 +575,10 @@ class TestToolFromFunctionOutputSchema:
         assert tool.output_schema == snapshot(
             {
                 "properties": {
-                    "name": {"title": "Name", "type": "string"},
-                    "age": {"title": "Age", "type": "integer"},
+                    "name": {"type": "string"},
+                    "age": {"type": "integer"},
                 },
                 "required": ["name", "age"],
-                "title": "Person",
                 "type": "object",
             }
         )
@@ -765,9 +758,8 @@ class TestToolFromFunctionOutputSchema:
         tool = Tool.from_function(func)
         assert tool.output_schema == snapshot(
             {
-                "properties": {"result": {"title": "Result", "type": "integer"}},
+                "properties": {"result": {"type": "integer"}},
                 "required": ["result"],
-                "title": "_WrappedResult",
                 "type": "object",
                 "x-fastmcp-wrap-result": True,
             }
@@ -838,9 +830,8 @@ class TestToolFromFunctionOutputSchema:
         tool = Tool.from_function(func)
         assert tool.output_schema == snapshot(
             {
-                "properties": {"result": {"title": "Result", "type": "string"}},
+                "properties": {"result": {"type": "string"}},
                 "required": ["result"],
-                "title": "_WrappedResult",
                 "type": "object",
                 "x-fastmcp-wrap-result": True,
             }
@@ -941,238 +932,215 @@ class TestToolFromFunctionOutputSchema:
                 Tool.from_function(func, output_schema=schema)
 
 
+class SampleModel(BaseModel):
+    x: int
+    y: str
+
+
 class TestConvertResultToContent:
     """Tests for the _convert_to_content helper function."""
 
-    def test_none_result(self):
-        """Test that None results in an empty list."""
-        result = _convert_to_content(None)
-        assert isinstance(result, list)
-        assert len(result) == 0
-
-    def test_text_content_result(self):
-        """Test that TextContent is returned as a list containing itself."""
-        content = TextContent(type="text", text="hello")
-        result = _convert_to_content(content)
-        assert isinstance(result, list)
-        assert len(result) == 1
-        assert result[0] is content
-
-    def test_image_content_result(self):
-        """Test that ImageContent is returned as a list containing itself."""
-        content = ImageContent(type="image", data="fakeimagedata", mimeType="image/png")
-        result = _convert_to_content(content)
-        assert isinstance(result, list)
-        assert len(result) == 1
-        assert result[0] is content
-
-    def test_embedded_resource_result(self):
-        """Test that EmbeddedResource is returned as a list containing itself."""
-        content = EmbeddedResource(
-            type="resource",
-            resource=TextResourceContents(
-                uri=AnyUrl("resource://test"),
-                mimeType="text/plain",
-                text="resource content",
+    @pytest.mark.parametrize(
+        argnames=("result", "expected"),
+        argvalues=[
+            (True, "true"),
+            ("hello", "hello"),
+            (123, "123"),
+            (123.45, "123.45"),
+            ({"key": "value"}, '{"key":"value"}'),
+            (
+                SampleModel(x=1, y="hello"),
+                '{"x":1,"y":"hello"}',
             ),
+        ],
+        ids=[
+            "boolean",
+            "string",
+            "integer",
+            "float",
+            "object",
+            "basemodel",
+        ],
+    )
+    def test_convert_singular(self, result, expected):
+        """Test that a single item is converted to a TextContent."""
+        converted = _convert_to_content(result)
+        assert converted == [TextContent(type="text", text=expected)]
+
+    @pytest.mark.parametrize(
+        argnames=("result", "expected_text"),
+        argvalues=[
+            ([None], "[null]"),
+            ([None, None], "[null,null]"),
+            ([True], "[true]"),
+            ([True, False], "[true,false]"),
+            (["hello"], '["hello"]'),
+            (["hello", "world"], '["hello","world"]'),
+            ([123], "[123]"),
+            ([123, 456], "[123,456]"),
+            ([123.45], "[123.45]"),
+            ([123.45, 456.78], "[123.45,456.78]"),
+            ([{"key": "value"}], '[{"key":"value"}]'),
+            (
+                [{"key": "value"}, {"key2": "value2"}],
+                '[{"key":"value"},{"key2":"value2"}]',
+            ),
+            ([SampleModel(x=1, y="hello")], '[{"x":1,"y":"hello"}]'),
+            (
+                [SampleModel(x=1, y="hello"), SampleModel(x=2, y="world")],
+                '[{"x":1,"y":"hello"},{"x":2,"y":"world"}]',
+            ),
+            ([1, "two", None, {"c": 3}, False], '[1,"two",null,{"c":3},false]'),
+        ],
+        ids=[
+            "none",
+            "none_many",
+            "boolean",
+            "boolean_many",
+            "string",
+            "string_many",
+            "integer",
+            "integer_many",
+            "float",
+            "float_many",
+            "object",
+            "object_many",
+            "basemodel",
+            "basemodel_many",
+            "mixed",
+        ],
+    )
+    def test_convert_list(self, result, expected_text):
+        """Test that a list is converted to a TextContent."""
+        converted = _convert_to_content(result)
+        assert converted == [TextContent(type="text", text=expected_text)]
+
+    @pytest.mark.parametrize(
+        argnames="content_block",
+        argvalues=[
+            (TextContent(type="text", text="hello")),
+            (ImageContent(type="image", data="fakeimagedata", mimeType="image/png")),
+            (AudioContent(type="audio", data="fakeaudiodata", mimeType="audio/mpeg")),
+            (
+                ResourceLink(
+                    type="resource_link",
+                    name="test resource",
+                    uri=AnyUrl("resource://test"),
+                )
+            ),
+            (
+                EmbeddedResource(
+                    type="resource",
+                    resource=TextResourceContents(
+                        uri=AnyUrl("resource://test"),
+                        mimeType="text/plain",
+                        text="resource content",
+                    ),
+                )
+            ),
+        ],
+        ids=["text", "image", "audio", "resource link", "embedded resource"],
+    )
+    def test_convert_content_block(self, content_block):
+        converted = _convert_to_content(content_block)
+        assert converted == [content_block]
+
+        converted = _convert_to_content([content_block, content_block])
+        assert converted == [content_block, content_block]
+
+    @pytest.mark.parametrize(
+        argnames=("result", "expected"),
+        argvalues=[
+            (
+                Image(data=b"fakeimagedata"),
+                [
+                    ImageContent(
+                        type="image", data="ZmFrZWltYWdlZGF0YQ==", mimeType="image/png"
+                    )
+                ],
+            ),
+            (
+                Audio(data=b"fakeaudiodata"),
+                [
+                    AudioContent(
+                        type="audio", data="ZmFrZWF1ZGlvZGF0YQ==", mimeType="audio/wav"
+                    )
+                ],
+            ),
+            (
+                File(data=b"filedata", format="octet-stream"),
+                [
+                    EmbeddedResource(
+                        type="resource",
+                        resource=BlobResourceContents(
+                            uri=AnyUrl("file:///resource.octet-stream"),
+                            blob="ZmlsZWRhdGE=",
+                            mimeType="application/octet-stream",
+                        ),
+                    )
+                ],
+            ),
+        ],
+        ids=["image", "audio", "file"],
+    )
+    def test_convert_helpers(self, result, expected):
+        converted = _convert_to_content(result)
+        assert converted == expected
+
+    def test_convert_mixed_content(self):
+        result = [
+            "hello",
+            123,
+            123.45,
+            {"key": "value"},
+            SampleModel(x=1, y="hello"),
+            Image(data=b"fakeimagedata"),
+            Audio(data=b"fakeaudiodata"),
+            ResourceLink(
+                type="resource_link",
+                name="test resource",
+                uri=AnyUrl("resource://test"),
+            ),
+            EmbeddedResource(
+                type="resource",
+                resource=TextResourceContents(
+                    uri=AnyUrl("resource://test"),
+                    mimeType="text/plain",
+                    text="resource content",
+                ),
+            ),
+        ]
+
+        converted = _convert_to_content(result)
+
+        assert converted == snapshot(
+            [
+                TextContent(type="text", text="hello"),
+                TextContent(type="text", text="123"),
+                TextContent(type="text", text="123.45"),
+                TextContent(type="text", text='{"key":"value"}'),
+                TextContent(type="text", text='{"x":1,"y":"hello"}'),
+                ImageContent(
+                    type="image", data="ZmFrZWltYWdlZGF0YQ==", mimeType="image/png"
+                ),
+                AudioContent(
+                    type="audio", data="ZmFrZWF1ZGlvZGF0YQ==", mimeType="audio/wav"
+                ),
+                ResourceLink(
+                    name="test resource",
+                    uri=AnyUrl("resource://test"),
+                    type="resource_link",
+                ),
+                EmbeddedResource(
+                    type="resource",
+                    resource=TextResourceContents(
+                        uri=AnyUrl("resource://test"),
+                        mimeType="text/plain",
+                        text="resource content",
+                    ),
+                ),
+            ]
         )
-        result = _convert_to_content(content)
-        assert isinstance(result, list)
-        assert len(result) == 1
-        assert result[0] is content
-
-    def test_image_object_result(self):
-        """Test that an Image object is converted to ImageContent."""
-        image_obj = Image(data=b"fakeimagedata")
-
-        result = _convert_to_content(image_obj)
-
-        assert isinstance(result, list)
-        assert len(result) == 1
-        assert isinstance(result[0], ImageContent)
-        assert result[0].data == "ZmFrZWltYWdlZGF0YQ=="
-
-    def test_audio_object_result(self):
-        """Test that an Audio object is converted to AudioContent."""
-        audio_obj = Audio(data=b"fakeaudiodata")
-
-        result = _convert_to_content(audio_obj)
-
-        assert isinstance(result, list)
-        assert len(result) == 1
-        assert isinstance(result[0], AudioContent)
-        assert result[0].data == "ZmFrZWF1ZGlvZGF0YQ=="
-
-    def test_file_object_result(self):
-        """Test that a File object is converted to EmbeddedResource with BlobResourceContents."""
-        file_obj = File(data=b"filedata", format="octet-stream")
-
-        result = _convert_to_content(file_obj)
-
-        assert isinstance(result, list)
-        assert len(result) == 1
-        assert isinstance(result[0], EmbeddedResource)
-        assert result[0].type == "resource"
-        assert hasattr(result[0], "resource")
-        resource = result[0].resource
-        assert resource.mimeType == "application/octet-stream"
-        # Check for blob attribute and its value
-        assert hasattr(resource, "blob")
-        assert getattr(resource, "blob") == "ZmlsZWRhdGE="  # base64 encoded "filedata"
-        # Convert URI to string for startswith check
-        assert str(resource.uri).startswith("file:///resource.octet-stream")
-
-    def test_file_object_text_result(self):
-        """Test that a File object with text data is converted to EmbeddedResource with TextResourceContents."""
-        file_obj = File(data=b"sometext", format="plain")
-        result = _convert_to_content(file_obj)
-        assert isinstance(result, list)
-        assert len(result) == 1
-        assert isinstance(result[0], EmbeddedResource)
-        assert result[0].type == "resource"
-        resource = result[0].resource
-        assert isinstance(resource, TextResourceContents)
-        assert resource.mimeType == "text/plain"
-        assert resource.text == "sometext"
-
-    def test_basic_type_result(self):
-        """Test that a basic type is converted to TextContent."""
-        result = _convert_to_content(123)
-        assert isinstance(result, list)
-        assert len(result) == 1
-        assert isinstance(result[0], TextContent)
-        assert result[0].text == "123"
-
-        result = _convert_to_content("hello")
-        assert isinstance(result, list)
-        assert len(result) == 1
-        assert isinstance(result[0], TextContent)
-        assert result[0].text == "hello"
-
-        result = _convert_to_content({"a": 1, "b": 2})
-        assert isinstance(result, list)
-        assert len(result) == 1
-        assert isinstance(result[0], TextContent)
-        assert result[0].text == '{"a":1,"b":2}'
-
-    def test_list_of_basic_types(self):
-        """Test that a list of basic types is converted to a single TextContent."""
-        result = _convert_to_content([1, "two", {"c": 3}])
-        assert isinstance(result, list)
-        assert len(result) == 1
-        assert isinstance(result[0], TextContent)
-        assert result[0].text == '[1,"two",{"c":3}]'
-
-    def test_list_of_mcp_types(self):
-        """Test that a list of MCP types is returned as a list of those types."""
-        content1 = TextContent(type="text", text="hello")
-        content2 = ImageContent(
-            type="image", data="fakeimagedata2", mimeType="image/png"
-        )
-        result = _convert_to_content([content1, content2])
-        assert isinstance(result, list)
-        assert len(result) == 2
-        assert result[0] is content1
-        assert result[1] is content2
-
-    def test_list_of_mixed_types(self):
-        """Test that a list of mixed types is converted correctly."""
-        content1 = TextContent(type="text", text="hello")
-        image_obj = Image(data=b"fakeimagedata")
-        basic_data = {"a": 1}
-        result = _convert_to_content([content1, image_obj, basic_data])
-
-        assert isinstance(result, list)
-        assert len(result) == 3
-
-        text_content_count = sum(isinstance(item, TextContent) for item in result)
-        image_content_count = sum(isinstance(item, ImageContent) for item in result)
-
-        assert text_content_count == 2
-        assert image_content_count == 1
-
-        text_item = next(item for item in result if isinstance(item, TextContent))
-        assert text_item.text == '[{"a":1}]'
-
-        image_item = next(item for item in result if isinstance(item, ImageContent))
-        assert image_item.data == "ZmFrZWltYWdlZGF0YQ=="
-
-    def test_list_of_mixed_types_list(self):
-        """Test that a list of mixed types, including a list as one of the elements, is converted correctly."""
-        content1 = TextContent(type="text", text="hello")
-        image_obj = Image(data=b"fakeimagedata")
-        basic_data = [{"a": 1}, {"b": 2}]
-        result = _convert_to_content([content1, image_obj, basic_data])
-
-        assert isinstance(result, list)
-        assert len(result) == 3
-
-        text_content_count = sum(isinstance(item, TextContent) for item in result)
-        image_content_count = sum(isinstance(item, ImageContent) for item in result)
-
-        assert text_content_count == 2
-        assert image_content_count == 1
-
-        text_item = next(item for item in result if isinstance(item, TextContent))
-        assert text_item.text == '[[{"a":1},{"b":2}]]'
-
-        image_item = next(item for item in result if isinstance(item, ImageContent))
-        assert image_item.data == "ZmFrZWltYWdlZGF0YQ=="
-
-    def test_list_of_mixed_types_with_audio(self):
-        """Test that a list of mixed types including Audio is converted correctly."""
-        content1 = TextContent(type="text", text="hello")
-        audio_obj = Audio(data=b"fakeaudiodata")
-        basic_data = {"a": 1}
-        result = _convert_to_content([content1, audio_obj, basic_data])
-
-        assert isinstance(result, list)
-        assert len(result) == 3
-
-        text_content_count = sum(isinstance(item, TextContent) for item in result)
-        audio_content_count = sum(isinstance(item, AudioContent) for item in result)
-
-        assert text_content_count == 2
-        assert audio_content_count == 1
-
-        text_item = next(item for item in result if isinstance(item, TextContent))
-        assert text_item.text == '[{"a":1}]'
-
-        audio_item = next(item for item in result if isinstance(item, AudioContent))
-        assert audio_item.data == "ZmFrZWF1ZGlvZGF0YQ=="
-
-    def test_list_of_mixed_types_with_file(self):
-        """Test that a list of mixed types including File is converted correctly."""
-        content1 = TextContent(type="text", text="hello")
-        file_obj = File(data=b"filedata", format="octet-stream")
-        basic_data = {"a": 1}
-        result = _convert_to_content([content1, file_obj, basic_data])
-
-        assert isinstance(result, list)
-        assert len(result) == 3
-
-        text_content_count = sum(isinstance(item, TextContent) for item in result)
-        embedded_content_count = sum(
-            isinstance(item, EmbeddedResource) and item.type == "resource"
-            for item in result
-        )
-
-        assert text_content_count == 2
-        assert embedded_content_count == 1
-
-        text_item = next(item for item in result if isinstance(item, TextContent))
-        assert text_item.text == '[{"a":1}]'
-
-        embedded_item = next(
-            item
-            for item in result
-            if isinstance(item, EmbeddedResource) and item.type == "resource"
-        )
-        resource = embedded_item.resource
-        assert resource.mimeType == "application/octet-stream"
-        # Check for blob attribute and its value
-        assert hasattr(resource, "blob")
-        assert getattr(resource, "blob") == "ZmlsZWRhdGE="
 
     def test_empty_list(self):
         """Test that an empty list results in an empty list."""
@@ -1188,17 +1156,17 @@ class TestConvertResultToContent:
         assert isinstance(result[0], TextContent)
         assert result[0].text == "{}"
 
-    def test_with_custom_serializer(self):
+    def test_custom_serializer(self):
         """Test that a custom serializer is used for non-MCP types."""
 
         def custom_serializer(data):
             return f"Serialized: {data}"
 
         result = _convert_to_content({"a": 1}, serializer=custom_serializer)
-        assert isinstance(result, list)
-        assert len(result) == 1
-        assert isinstance(result[0], TextContent)
-        assert result[0].text == "Serialized: {'a': 1}"
+
+        assert result == snapshot(
+            [TextContent(type="text", text="Serialized: {'a': 1}")]
+        )
 
     def test_custom_serializer_error_fallback(self, caplog):
         """Test that if a custom serializer fails, it falls back to the default."""
@@ -1212,56 +1180,10 @@ class TestConvertResultToContent:
             )
 
         assert isinstance(result, list)
-        assert len(result) == 1
-        assert isinstance(result[0], TextContent)
-        # Should fall back to default serializer (pydantic_core.to_json)
-        assert json.loads(result[0].text) == {"a": 1}
+        assert result == snapshot([TextContent(type="text", text='{"a":1}')])
+
         assert "Error serializing tool result" in caplog.text
 
-    def test_process_as_single_item_flag(self):
-        """Test that _process_as_single_item forces list to be treated as one item."""
-
-        result = _convert_to_content([1, "two", {"c": 3}], _process_as_single_item=True)
-        assert isinstance(result, list)
-        assert len(result) == 1
-        assert isinstance(result[0], TextContent)
-        assert result[0].text == '[1,"two",{"c":3}]'
-
-        content1 = TextContent(type="text", text="hello")
-        result = _convert_to_content([1, content1], _process_as_single_item=True)
-        assert isinstance(result, list)
-        assert len(result) == 1
-        assert isinstance(result[0], TextContent)
-
-        assert json.loads(result[0].text) == [
-            1,
-            {"type": "text", "text": "hello", "annotations": None, "_meta": None},
-        ]
-
-    def test_single_element_list_preserves_structure(self):
-        """Test that single-element lists preserve their list structure."""
-
-        # Test with a single integer
-        result = _convert_to_content([1])
-        assert isinstance(result, list)
-        assert len(result) == 1
-        assert isinstance(result[0], TextContent)
-        assert result[0].text == "[1]"  # Should be "[1]", not "1"
-
-        # Test with a single string
-        result = _convert_to_content(["hello"])
-        assert isinstance(result, list)
-        assert len(result) == 1
-        assert isinstance(result[0], TextContent)
-        assert result[0].text == '["hello"]'  # Should be ["hello"], not "hello"
-
-        # Test with a single dict
-        result = _convert_to_content([{"a": 1}])
-        assert isinstance(result, list)
-        assert len(result) == 1
-        assert isinstance(result[0], TextContent)
-        assert result[0].text == '[{"a":1}]'  # Should be wrapped in a list
-
 
 class TestAutomaticStructuredContent:
     """Tests for automatic structured content generation based on return types."""
@@ -1385,9 +1307,30 @@ class TestAutomaticStructuredContent:
 
         result = await tool.run({})
 
-        # Should only have content, no structured content
-        assert len(result.content) == 1
-        assert isinstance(result.content[0], TextContent)
+        assert result.structured_content is None
+        assert result.content == snapshot(
+            [TextContent(type="text", text="[1,2,3,4,5]")]
+        )
+
+    async def test_audio_return_creates_no_structured_content(self):
+        """Test that audio returns don't create structured content."""
+
+        def get_audio() -> AudioContent:
+            """No return annotation."""
+            return Audio(data=b"fakeaudiodata").to_audio_content()
+
+        # No output schema
+        tool = Tool.from_function(get_audio)
+
+        result = await tool.run({})
+
+        assert result.content == snapshot(
+            [
+                AudioContent(
+                    type="audio", data="ZmFrZWF1ZGlvZGF0YQ==", mimeType="audio/wav"
+                )
+            ]
+        )
         assert result.structured_content is None
 
     async def test_int_return_with_schema_creates_structured_content(self):
@@ -1452,8 +1395,8 @@ class TestAutomaticStructuredContent:
         async with Client(mcp) as client:
             result = await client.call_tool("get_profile", {"user_id": "456"})
 
-            # Client should deserialize back to a dataclass (type name preserved with new compression)
-            assert result.data.__class__.__name__ == "UserProfile"
+            # Client should deserialize back to a dataclass (but type name is lost with title pruning)
+            assert result.data.__class__.__name__ == "Root"
             assert result.data.name == "Bob"
             assert result.data.age == 25
             assert result.data.verified is True
